# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtGui

from openlp.core.lib import build_icon, translate, SpellTextEdit
from openlp.core.lib.ui import create_button_box, UiStrings
from openlp.plugins.songs.lib import VerseType


class Ui_EditVerseDialog(object):
    def setupUi(self, editVerseDialog):
        editVerseDialog.setObjectName(u'editVerseDialog')
        editVerseDialog.resize(400, 400)
        editVerseDialog.setModal(True)
        editVerseDialog.setWindowIcon(build_icon(u':/icon/openlp-logo.svg'))
        self.dialogLayout = QtGui.QVBoxLayout(editVerseDialog)
        self.dialogLayout.setObjectName(u'dialogLayout')
        self.verseTextEdit = SpellTextEdit(editVerseDialog)
        self.verseTextEdit.setObjectName(u'verseTextEdit')
        self.dialogLayout.addWidget(self.verseTextEdit)
        self.verseTypeLayout = QtGui.QHBoxLayout()
        self.verseTypeLayout.setObjectName(u'verseTypeLayout')
        self.splitButton = QtGui.QPushButton(editVerseDialog)
        self.splitButton.setIcon(build_icon(u':/general/general_add.png'))
        self.splitButton.setObjectName(u'splitButton')
        self.verseTypeLayout.addWidget(self.splitButton)
        self.verseTypeLabel = QtGui.QLabel(editVerseDialog)
        self.verseTypeLabel.setObjectName(u'verseTypeLabel')
        self.verseTypeLayout.addWidget(self.verseTypeLabel)
        self.verseTypeComboBox = QtGui.QComboBox(editVerseDialog)
        self.verseTypeComboBox.addItems([u'', u'', u'', u'', u'', u'', u''])
        self.verseTypeComboBox.setObjectName(u'verseTypeComboBox')
        self.verseTypeLabel.setBuddy(self.verseTypeComboBox)
        self.verseTypeLayout.addWidget(self.verseTypeComboBox)
        self.verseNumberBox = QtGui.QSpinBox(editVerseDialog)
        self.verseNumberBox.setMinimum(1)
        self.verseNumberBox.setObjectName(u'verseNumberBox')
        self.verseTypeLayout.addWidget(self.verseNumberBox)
        self.insertButton = QtGui.QPushButton(editVerseDialog)
        self.insertButton.setIcon(build_icon(u':/general/general_add.png'))
        self.insertButton.setObjectName(u'insertButton')
        self.verseTypeLayout.addWidget(self.insertButton)
        self.verseTypeLayout.addStretch()
        self.dialogLayout.addLayout(self.verseTypeLayout)
        self.buttonBox = create_button_box(editVerseDialog, u'buttonBox',
            [u'cancel', u'ok'])
        self.dialogLayout.addWidget(self.buttonBox)
        self.retranslateUi(editVerseDialog)

    def retranslateUi(self, editVerseDialog):
        editVerseDialog.setWindowTitle(
            translate('SongsPlugin.EditVerseForm', 'Edit Verse'))
        self.verseTypeLabel.setText(
            translate('SongsPlugin.EditVerseForm', '&Verse type:'))
        self.verseTypeComboBox.setItemText(VerseType.Verse,
            VerseType.TranslatedNames[VerseType.Verse])
        self.verseTypeComboBox.setItemText(VerseType.Chorus,
            VerseType.TranslatedNames[VerseType.Chorus])
        self.verseTypeComboBox.setItemText(VerseType.Bridge,
            VerseType.TranslatedNames[VerseType.Bridge])
        self.verseTypeComboBox.setItemText(VerseType.PreChorus,
            VerseType.TranslatedNames[VerseType.PreChorus])
        self.verseTypeComboBox.setItemText(VerseType.Intro,
            VerseType.TranslatedNames[VerseType.Intro])
        self.verseTypeComboBox.setItemText(VerseType.Ending,
            VerseType.TranslatedNames[VerseType.Ending])
        self.verseTypeComboBox.setItemText(VerseType.Other,
            VerseType.TranslatedNames[VerseType.Other])
        self.splitButton.setText(UiStrings().Split)
        self.splitButton.setToolTip(UiStrings().SplitToolTip)
        self.insertButton.setText(
            translate('SongsPlugin.EditVerseForm', '&Insert'))
        self.insertButton.setToolTip(
            translate('SongsPlugin.EditVerseForm', 'Split a slide into two '
            'by inserting a verse splitter.'))



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

import logging

from openlp.core.lib import Plugin, StringContent, build_icon, translate, \
    Receiver, ImageSource
from openlp.core.lib.settings import Settings
from openlp.plugins.images.lib import ImageMediaItem, ImageTab

log = logging.getLogger(__name__)

class ImagePlugin(Plugin):
    log.info(u'Image Plugin loaded')

    def __init__(self, plugin_helpers):
        Plugin.__init__(self, u'images', plugin_helpers, ImageMediaItem,
            ImageTab)
        self.weight = -7
        self.iconPath = u':/plugins/plugin_images.png'
        self.icon = build_icon(self.iconPath)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'image_updated'), self.image_updated)

    def about(self):
        about_text = translate('ImagePlugin', '<strong>Image Plugin</strong>'
            '<br />The image plugin provides displaying of images.<br />One '
            'of the distinguishing features of this plugin is the ability to '
            'group a number of images together in the service manager, making '
            'the displaying of multiple images easier. This plugin can also '
            'make use of OpenLP\'s "timed looping" feature to create a slide '
            'show that runs automatically. In addition to this, images from '
            'the plugin can be used to override the current theme\'s '
            'background, which renders text-based items like songs with the '
            'selected image as a background instead of the background '
            'provided by the theme.')
        return about_text

    def setPluginTextStrings(self):
        """
        Called to define all translatable texts of the plugin
        """
        ## Name PluginList ##
        self.textStrings[StringContent.Name] = {
            u'singular': translate('ImagePlugin', 'Image', 'name singular'),
            u'plural': translate('ImagePlugin', 'Images', 'name plural')
        }
        ## Name for MediaDockManager, SettingsManager ##
        self.textStrings[StringContent.VisibleName] = {
            u'title': translate('ImagePlugin', 'Images', 'container title')
        }
        # Middle Header Bar
        tooltips = {
            u'load': translate('ImagePlugin', 'Load a new image.'),
            u'import': u'',
            u'new': translate('ImagePlugin', 'Add a new image.'),
            u'edit': translate('ImagePlugin', 'Edit the selected image.'),
            u'delete': translate('ImagePlugin', 'Delete the selected image.'),
            u'preview': translate('ImagePlugin', 'Preview the selected image.'),
            u'live': translate('ImagePlugin', 'Send the selected image live.'),
            u'service': translate('ImagePlugin',
                'Add the selected image to the service.')
        }
        self.setPluginUiTextStrings(tooltips)

    def image_updated(self):
        """
        Triggered by saving and changing the image border.  Sets the images in
        image manager to require updates.  Actual update is triggered by the
        last part of saving the config.
        """
        background = QtGui.QColor(Settings().value(self.settingsSection
            + u'/background color', QtCore.QVariant(u'#000000')))
        self.liveController.imageManager.updateImagesBorder(
            ImageSource.ImagePlugin, background)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The bible import functions for OpenLP
"""
import logging
import os

from PyQt4 import QtCore, QtGui

from openlp.core.lib import Receiver, translate
from openlp.core.lib.db import delete_database
from openlp.core.lib.ui import UiStrings, critical_error_message_box
from openlp.core.lib.settings import Settings
from openlp.core.ui.wizard import OpenLPWizard, WizardStrings
from openlp.core.utils import AppLocation, locale_compare
from openlp.plugins.bibles.lib.manager import BibleFormat
from openlp.plugins.bibles.lib.db import BiblesResourcesDB, clean_filename

log = logging.getLogger(__name__)


class WebDownload(object):
    """
    Provides an enumeration for the web bible types available to OpenLP.
    """
    Unknown = -1
    Crosswalk = 0
    BibleGateway = 1
    Bibleserver = 2

    Names = [u'Crosswalk', u'BibleGateway', u'Bibleserver']


class BibleImportForm(OpenLPWizard):
    """
    This is the Bible Import Wizard, which allows easy importing of Bibles
    into OpenLP from other formats like OSIS, CSV and OpenSong.
    """
    log.info(u'BibleImportForm loaded')

    def __init__(self, parent, manager, bibleplugin):
        """
        Instantiate the wizard, and run any extra setup we need to.

        ``parent``
            The QWidget-derived parent of the wizard.

        ``manager``
            The Bible manager.

        ``bibleplugin``
            The Bible plugin.
        """
        self.manager = manager
        self.web_bible_list = {}
        OpenLPWizard.__init__(self, parent, bibleplugin, u'bibleImportWizard',
            u':/wizards/wizard_importbible.bmp')

    def setupUi(self, image):
        """
        Set up the UI for the bible wizard.
        """
        OpenLPWizard.setupUi(self, image)
        QtCore.QObject.connect(self.formatComboBox,
            QtCore.SIGNAL(u'currentIndexChanged(int)'),
            self.onCurrentIndexChanged)

    def onCurrentIndexChanged(self, index):
        """
        Called when the format combo box's index changed. We have to check if
        the import is available and accordingly to disable or enable the next
        button.
        """
        self.selectStack.setCurrentIndex(index)
        next_button = self.button(QtGui.QWizard.NextButton)
        next_button.setEnabled(BibleFormat.get_availability(index))

    def customInit(self):
        """
        Perform any custom initialisation for bible importing.
        """
        if BibleFormat.get_availability(BibleFormat.OpenLP1):
            self.openlp1DisabledLabel.hide()
        else:
            self.openlp1FileLabel.hide()
            self.openlp1FileEdit.hide()
            self.openlp1BrowseButton.hide()
        self.manager.set_process_dialog(self)
        self.loadWebBibles()
        self.restart()
        self.selectStack.setCurrentIndex(0)

    def customSignals(self):
        """
        Set up the signals used in the bible importer.
        """
        QtCore.QObject.connect(self.webSourceComboBox,
            QtCore.SIGNAL(u'currentIndexChanged(int)'),
            self.onWebSourceComboBoxIndexChanged)
        QtCore.QObject.connect(self.osisBrowseButton,
            QtCore.SIGNAL(u'clicked()'),
            self.onOsisBrowseButtonClicked)
        QtCore.QObject.connect(self.csvBooksButton,
            QtCore.SIGNAL(u'clicked()'),
            self.onCsvBooksBrowseButtonClicked)
        QtCore.QObject.connect(self.csvVersesButton,
            QtCore.SIGNAL(u'clicked()'),
            self.onCsvVersesBrowseButtonClicked)
        QtCore.QObject.connect(self.openSongBrowseButton,
            QtCore.SIGNAL(u'clicked()'),
            self.onOpenSongBrowseButtonClicked)
        QtCore.QObject.connect(self.openlp1BrowseButton,
            QtCore.SIGNAL(u'clicked()'),
            self.onOpenlp1BrowseButtonClicked)

    def addCustomPages(self):
        """
        Add the bible import specific wizard pages.
        """
        # Select Page
        self.selectPage = QtGui.QWizardPage()
        self.selectPage.setObjectName(u'SelectPage')
        self.selectPageLayout = QtGui.QVBoxLayout(self.selectPage)
        self.selectPageLayout.setObjectName(u'SelectPageLayout')
        self.formatLayout = QtGui.QFormLayout()
        self.formatLayout.setObjectName(u'FormatLayout')
        self.formatLabel = QtGui.QLabel(self.selectPage)
        self.formatLabel.setObjectName(u'FormatLabel')
        self.formatComboBox = QtGui.QComboBox(self.selectPage)
        self.formatComboBox.addItems([u'', u'', u'', u'', u''])
        self.formatComboBox.setObjectName(u'FormatComboBox')
        self.formatLayout.addRow(self.formatLabel, self.formatComboBox)
        self.spacer = QtGui.QSpacerItem(10, 0, QtGui.QSizePolicy.Fixed,
            QtGui.QSizePolicy.Minimum)
        self.formatLayout.setItem(1, QtGui.QFormLayout.LabelRole, self.spacer)
        self.selectPageLayout.addLayout(self.formatLayout)
        self.selectStack = QtGui.QStackedLayout()
        self.selectStack.setObjectName(u'SelectStack')
        self.osisWidget = QtGui.QWidget(self.selectPage)
        self.osisWidget.setObjectName(u'OsisWidget')
        self.osisLayout = QtGui.QFormLayout(self.osisWidget)
        self.osisLayout.setMargin(0)
        self.osisLayout.setObjectName(u'OsisLayout')
        self.osisFileLabel = QtGui.QLabel(self.osisWidget)
        self.osisFileLabel.setObjectName(u'OsisFileLabel')
        self.osisFileLayout = QtGui.QHBoxLayout()
        self.osisFileLayout.setObjectName(u'OsisFileLayout')
        self.osisFileEdit = QtGui.QLineEdit(self.osisWidget)
        self.osisFileEdit.setObjectName(u'OsisFileEdit')
        self.osisFileLayout.addWidget(self.osisFileEdit)
        self.osisBrowseButton = QtGui.QToolButton(self.osisWidget)
        self.osisBrowseButton.setIcon(self.openIcon)
        self.osisBrowseButton.setObjectName(u'OsisBrowseButton')
        self.osisFileLayout.addWidget(self.osisBrowseButton)
        self.osisLayout.addRow(self.osisFileLabel, self.osisFileLayout)
        self.osisLayout.setItem(1, QtGui.QFormLayout.LabelRole, self.spacer)
        self.selectStack.addWidget(self.osisWidget)
        self.csvWidget = QtGui.QWidget(self.selectPage)
        self.csvWidget.setObjectName(u'CsvWidget')
        self.csvLayout = QtGui.QFormLayout(self.csvWidget)
        self.csvLayout.setMargin(0)
        self.csvLayout.setObjectName(u'CsvLayout')
        self.csvBooksLabel = QtGui.QLabel(self.csvWidget)
        self.csvBooksLabel.setObjectName(u'CsvBooksLabel')
        self.csvBooksLayout = QtGui.QHBoxLayout()
        self.csvBooksLayout.setObjectName(u'CsvBooksLayout')
        self.csvBooksEdit = QtGui.QLineEdit(self.csvWidget)
        self.csvBooksEdit.setObjectName(u'CsvBooksEdit')
        self.csvBooksLayout.addWidget(self.csvBooksEdit)
        self.csvBooksButton = QtGui.QToolButton(self.csvWidget)
        self.csvBooksButton.setIcon(self.openIcon)
        self.csvBooksButton.setObjectName(u'CsvBooksButton')
        self.csvBooksLayout.addWidget(self.csvBooksButton)
        self.csvLayout.addRow(self.csvBooksLabel, self.csvBooksLayout)
        self.csvVersesLabel = QtGui.QLabel(self.csvWidget)
        self.csvVersesLabel.setObjectName(u'CsvVersesLabel')
        self.csvVersesLayout = QtGui.QHBoxLayout()
        self.csvVersesLayout.setObjectName(u'CsvVersesLayout')
        self.csvVersesEdit = QtGui.QLineEdit(self.csvWidget)
        self.csvVersesEdit.setObjectName(u'CsvVersesEdit')
        self.csvVersesLayout.addWidget(self.csvVersesEdit)
        self.csvVersesButton = QtGui.QToolButton(self.csvWidget)
        self.csvVersesButton.setIcon(self.openIcon)
        self.csvVersesButton.setObjectName(u'CsvVersesButton')
        self.csvVersesLayout.addWidget(self.csvVersesButton)
        self.csvLayout.addRow(self.csvVersesLabel, self.csvVersesLayout)
        self.csvLayout.setItem(3, QtGui.QFormLayout.LabelRole, self.spacer)
        self.selectStack.addWidget(self.csvWidget)
        self.openSongWidget = QtGui.QWidget(self.selectPage)
        self.openSongWidget.setObjectName(u'OpenSongWidget')
        self.openSongLayout = QtGui.QFormLayout(self.openSongWidget)
        self.openSongLayout.setMargin(0)
        self.openSongLayout.setObjectName(u'OpenSongLayout')
        self.openSongFileLabel = QtGui.QLabel(self.openSongWidget)
        self.openSongFileLabel.setObjectName(u'OpenSongFileLabel')
        self.openSongFileLayout = QtGui.QHBoxLayout()
        self.openSongFileLayout.setObjectName(u'OpenSongFileLayout')
        self.openSongFileEdit = QtGui.QLineEdit(self.openSongWidget)
        self.openSongFileEdit.setObjectName(u'OpenSongFileEdit')
        self.openSongFileLayout.addWidget(self.openSongFileEdit)
        self.openSongBrowseButton = QtGui.QToolButton(self.openSongWidget)
        self.openSongBrowseButton.setIcon(self.openIcon)
        self.openSongBrowseButton.setObjectName(u'OpenSongBrowseButton')
        self.openSongFileLayout.addWidget(self.openSongBrowseButton)
        self.openSongLayout.addRow(self.openSongFileLabel,
            self.openSongFileLayout)
        self.openSongLayout.setItem(1, QtGui.QFormLayout.LabelRole, self.spacer)
        self.selectStack.addWidget(self.openSongWidget)
        self.webTabWidget = QtGui.QTabWidget(self.selectPage)
        self.webTabWidget.setObjectName(u'WebTabWidget')
        self.webBibleTab = QtGui.QWidget()
        self.webBibleTab.setObjectName(u'WebBibleTab')
        self.webBibleLayout = QtGui.QFormLayout(self.webBibleTab)
        self.webBibleLayout.setObjectName(u'WebBibleLayout')
        self.webSourceLabel = QtGui.QLabel(self.webBibleTab)
        self.webSourceLabel.setObjectName(u'WebSourceLabel')
        self.webBibleLayout.setWidget(0, QtGui.QFormLayout.LabelRole,
            self.webSourceLabel)
        self.webSourceComboBox = QtGui.QComboBox(self.webBibleTab)
        self.webSourceComboBox.setObjectName(u'WebSourceComboBox')
        self.webSourceComboBox.addItems([u'', u'', u''])
        self.webBibleLayout.setWidget(0, QtGui.QFormLayout.FieldRole,
            self.webSourceComboBox)
        self.webTranslationLabel = QtGui.QLabel(self.webBibleTab)
        self.webTranslationLabel.setObjectName(u'webTranslationLabel')
        self.webBibleLayout.setWidget(1, QtGui.QFormLayout.LabelRole,
            self.webTranslationLabel)
        self.webTranslationComboBox = QtGui.QComboBox(self.webBibleTab)
        self.webTranslationComboBox.setSizeAdjustPolicy(
            QtGui.QComboBox.AdjustToContents)
        self.webTranslationComboBox.setObjectName(u'WebTranslationComboBox')
        self.webBibleLayout.setWidget(1, QtGui.QFormLayout.FieldRole,
            self.webTranslationComboBox)
        self.webTabWidget.addTab(self.webBibleTab, u'')
        self.webProxyTab = QtGui.QWidget()
        self.webProxyTab.setObjectName(u'WebProxyTab')
        self.webProxyLayout = QtGui.QFormLayout(self.webProxyTab)
        self.webProxyLayout.setObjectName(u'WebProxyLayout')
        self.webServerLabel = QtGui.QLabel(self.webProxyTab)
        self.webServerLabel.setObjectName(u'WebServerLabel')
        self.webProxyLayout.setWidget(0, QtGui.QFormLayout.LabelRole,
            self.webServerLabel)
        self.webServerEdit = QtGui.QLineEdit(self.webProxyTab)
        self.webServerEdit.setObjectName(u'WebServerEdit')
        self.webProxyLayout.setWidget(0, QtGui.QFormLayout.FieldRole,
            self.webServerEdit)
        self.webUserLabel = QtGui.QLabel(self.webProxyTab)
        self.webUserLabel.setObjectName(u'WebUserLabel')
        self.webProxyLayout.setWidget(1, QtGui.QFormLayout.LabelRole,
            self.webUserLabel)
        self.webUserEdit = QtGui.QLineEdit(self.webProxyTab)
        self.webUserEdit.setObjectName(u'WebUserEdit')
        self.webProxyLayout.setWidget(1, QtGui.QFormLayout.FieldRole,
            self.webUserEdit)
        self.webPasswordLabel = QtGui.QLabel(self.webProxyTab)
        self.webPasswordLabel.setObjectName(u'WebPasswordLabel')
        self.webProxyLayout.setWidget(2, QtGui.QFormLayout.LabelRole,
            self.webPasswordLabel)
        self.webPasswordEdit = QtGui.QLineEdit(self.webProxyTab)
        self.webPasswordEdit.setObjectName(u'WebPasswordEdit')
        self.webProxyLayout.setWidget(2, QtGui.QFormLayout.FieldRole,
            self.webPasswordEdit)
        self.webTabWidget.addTab(self.webProxyTab, u'')
        self.selectStack.addWidget(self.webTabWidget)
        self.openlp1Widget = QtGui.QWidget(self.selectPage)
        self.openlp1Widget.setObjectName(u'Openlp1Widget')
        self.openlp1Layout = QtGui.QFormLayout(self.openlp1Widget)
        self.openlp1Layout.setMargin(0)
        self.openlp1Layout.setObjectName(u'Openlp1Layout')
        self.openlp1FileLabel = QtGui.QLabel(self.openlp1Widget)
        self.openlp1FileLabel.setObjectName(u'Openlp1FileLabel')
        self.openlp1FileLayout = QtGui.QHBoxLayout()
        self.openlp1FileLayout.setObjectName(u'Openlp1FileLayout')
        self.openlp1FileEdit = QtGui.QLineEdit(self.openlp1Widget)
        self.openlp1FileEdit.setObjectName(u'Openlp1FileEdit')
        self.openlp1FileLayout.addWidget(self.openlp1FileEdit)
        self.openlp1BrowseButton = QtGui.QToolButton(self.openlp1Widget)
        self.openlp1BrowseButton.setIcon(self.openIcon)
        self.openlp1BrowseButton.setObjectName(u'Openlp1BrowseButton')
        self.openlp1FileLayout.addWidget(self.openlp1BrowseButton)
        self.openlp1Layout.addRow(self.openlp1FileLabel, self.openlp1FileLayout)
        self.openlp1DisabledLabel = QtGui.QLabel(self.openlp1Widget)
        self.openlp1DisabledLabel.setWordWrap(True)
        self.openlp1DisabledLabel.setObjectName(u'Openlp1DisabledLabel')
        self.openlp1Layout.addRow(self.openlp1DisabledLabel)
        self.openlp1Layout.setItem(1, QtGui.QFormLayout.LabelRole, self.spacer)
        self.selectStack.addWidget(self.openlp1Widget)
        self.selectPageLayout.addLayout(self.selectStack)
        self.addPage(self.selectPage)
        # License Page
        self.licenseDetailsPage = QtGui.QWizardPage()
        self.licenseDetailsPage.setObjectName(u'LicenseDetailsPage')
        self.licenseDetailsLayout = QtGui.QFormLayout(self.licenseDetailsPage)
        self.licenseDetailsLayout.setObjectName(u'LicenseDetailsLayout')
        self.versionNameLabel = QtGui.QLabel(self.licenseDetailsPage)
        self.versionNameLabel.setObjectName(u'VersionNameLabel')
        self.licenseDetailsLayout.setWidget(0, QtGui.QFormLayout.LabelRole,
            self.versionNameLabel)
        self.versionNameEdit = QtGui.QLineEdit(self.licenseDetailsPage)
        self.versionNameEdit.setObjectName(u'VersionNameEdit')
        self.licenseDetailsLayout.setWidget(0, QtGui.QFormLayout.FieldRole,
            self.versionNameEdit)
        self.copyrightLabel = QtGui.QLabel(self.licenseDetailsPage)
        self.copyrightLabel.setObjectName(u'CopyrightLabel')
        self.licenseDetailsLayout.setWidget(1, QtGui.QFormLayout.LabelRole,
            self.copyrightLabel)
        self.copyrightEdit = QtGui.QLineEdit(self.licenseDetailsPage)
        self.copyrightEdit.setObjectName(u'CopyrightEdit')
        self.licenseDetailsLayout.setWidget(1, QtGui.QFormLayout.FieldRole,
            self.copyrightEdit)
        self.permissionsLabel = QtGui.QLabel(self.licenseDetailsPage)
        self.permissionsLabel.setObjectName(u'PermissionsLabel')
        self.licenseDetailsLayout.setWidget(2, QtGui.QFormLayout.LabelRole,
            self.permissionsLabel)
        self.permissionsEdit = QtGui.QLineEdit(self.licenseDetailsPage)
        self.permissionsEdit.setObjectName(u'PermissionsEdit')
        self.licenseDetailsLayout.setWidget(2, QtGui.QFormLayout.FieldRole,
            self.permissionsEdit)
        self.addPage(self.licenseDetailsPage)

    def retranslateUi(self):
        """
        Allow for localisation of the bible import wizard.
        """
        self.setWindowTitle(
            translate('BiblesPlugin.ImportWizardForm', 'Bible Import Wizard'))
        self.titleLabel.setText(WizardStrings.HeaderStyle %
            translate('OpenLP.Ui', 'Welcome to the Bible Import Wizard'))
        self.informationLabel.setText(
            translate('BiblesPlugin.ImportWizardForm',
            'This wizard will help you to import Bibles from a variety of '
            'formats. Click the next button below to start the process by '
            'selecting a format to import from.'))
        self.selectPage.setTitle(WizardStrings.ImportSelect)
        self.selectPage.setSubTitle(WizardStrings.ImportSelectLong)
        self.formatLabel.setText(WizardStrings.FormatLabel)
        self.formatComboBox.setItemText(BibleFormat.OSIS, WizardStrings.OSIS)
        self.formatComboBox.setItemText(BibleFormat.CSV, WizardStrings.CSV)
        self.formatComboBox.setItemText(BibleFormat.OpenSong, WizardStrings.OS)
        self.formatComboBox.setItemText(BibleFormat.WebDownload,
            translate('BiblesPlugin.ImportWizardForm', 'Web Download'))
        self.formatComboBox.setItemText(BibleFormat.OpenLP1, UiStrings().OLPV1)
        self.openlp1FileLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Bible file:'))
        self.osisFileLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Bible file:'))
        self.csvBooksLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Books file:'))
        self.csvVersesLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Verses file:'))
        self.openSongFileLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Bible file:'))
        self.webSourceLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Location:'))
        self.webSourceComboBox.setItemText(WebDownload.Crosswalk,
            translate('BiblesPlugin.ImportWizardForm', 'Crosswalk'))
        self.webSourceComboBox.setItemText(WebDownload.BibleGateway,
            translate('BiblesPlugin.ImportWizardForm', 'BibleGateway'))
        self.webSourceComboBox.setItemText(WebDownload.Bibleserver,
            translate('BiblesPlugin.ImportWizardForm', 'Bibleserver'))
        self.webTranslationLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Bible:'))
        self.webTabWidget.setTabText(
            self.webTabWidget.indexOf(self.webBibleTab),
            translate('BiblesPlugin.ImportWizardForm', 'Download Options'))
        self.webServerLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Server:'))
        self.webUserLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Username:'))
        self.webPasswordLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Password:'))
        self.webTabWidget.setTabText(
            self.webTabWidget.indexOf(self.webProxyTab),
            translate('BiblesPlugin.ImportWizardForm',
            'Proxy Server (Optional)'))
        self.licenseDetailsPage.setTitle(
            translate('BiblesPlugin.ImportWizardForm', 'License Details'))
        self.licenseDetailsPage.setSubTitle(
            translate('BiblesPlugin.ImportWizardForm',
            'Set up the Bible\'s license details.'))
        self.versionNameLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Version name:'))
        self.copyrightLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Copyright:'))
        self.permissionsLabel.setText(
            translate('BiblesPlugin.ImportWizardForm', 'Permissions:'))
        self.progressPage.setTitle(WizardStrings.Importing)
        self.progressPage.setSubTitle(
            translate('BiblesPlugin.ImportWizardForm',
            'Please wait while your Bible is imported.'))
        self.progressLabel.setText(WizardStrings.Ready)
        self.progressBar.setFormat(u'%p%')
        self.openlp1DisabledLabel.setText(WizardStrings.NoSqlite)
        # Align all QFormLayouts towards each other.
        labelWidth = max(self.formatLabel.minimumSizeHint().width(),
            self.osisFileLabel.minimumSizeHint().width(),
            self.csvBooksLabel.minimumSizeHint().width(),
            self.csvVersesLabel.minimumSizeHint().width(),
            self.openSongFileLabel.minimumSizeHint().width(),
            self.openlp1FileLabel.minimumSizeHint().width())
        self.spacer.changeSize(labelWidth, 0,
            QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)

    def validateCurrentPage(self):
        """
        Validate the current page before moving on to the next page.
        """
        if self.currentPage() == self.welcomePage:
            return True
        elif self.currentPage() == self.selectPage:
            if self.field(u'source_format').toInt()[0] == BibleFormat.OSIS:
                if not self.field(u'osis_location').toString():
                    critical_error_message_box(UiStrings().NFSs,
                        WizardStrings.YouSpecifyFile % WizardStrings.OSIS)
                    self.osisFileEdit.setFocus()
                    return False
            elif self.field(u'source_format').toInt()[0] == BibleFormat.CSV:
                if not self.field(u'csv_booksfile').toString():
                    critical_error_message_box(UiStrings().NFSs,
                        translate('BiblesPlugin.ImportWizardForm',
                        'You need to specify a file with books of '
                        'the Bible to use in the import.'))
                    self.csvBooksEdit.setFocus()
                    return False
                elif not self.field(u'csv_versefile').toString():
                    critical_error_message_box(UiStrings().NFSs,
                        translate('BiblesPlugin.ImportWizardForm',
                        'You need to specify a file of Bible '
                        'verses to import.'))
                    self.csvVersesEdit.setFocus()
                    return False
            elif self.field(u'source_format').toInt()[0] == \
                BibleFormat.OpenSong:
                if not self.field(u'opensong_file').toString():
                    critical_error_message_box(UiStrings().NFSs,
                        WizardStrings.YouSpecifyFile % WizardStrings.OS)
                    self.openSongFileEdit.setFocus()
                    return False
            elif self.field(u'source_format').toInt()[0] == \
                BibleFormat.WebDownload:
                self.versionNameEdit.setText(
                    self.webTranslationComboBox.currentText())
                return True
            elif self.field(u'source_format').toInt()[0] == BibleFormat.OpenLP1:
                if not self.field(u'openlp1_location').toString():
                    critical_error_message_box(UiStrings().NFSs,
                        WizardStrings.YouSpecifyFile % UiStrings().OLPV1)
                    self.openlp1FileEdit.setFocus()
                    return False
            return True
        elif self.currentPage() == self.licenseDetailsPage:
            license_version = unicode(self.field(u'license_version').toString())
            license_copyright = \
                unicode(self.field(u'license_copyright').toString())
            path = AppLocation.get_section_data_path(u'bibles')
            if not license_version:
                critical_error_message_box(UiStrings().EmptyField,
                    translate('BiblesPlugin.ImportWizardForm',
                    'You need to specify a version name for your Bible.'))
                self.versionNameEdit.setFocus()
                return False
            elif not license_copyright:
                critical_error_message_box(UiStrings().EmptyField,
                    translate('BiblesPlugin.ImportWizardForm',
                    'You need to set a copyright for your Bible. '
                    'Bibles in the Public Domain need to be marked as such.'))
                self.copyrightEdit.setFocus()
                return False
            elif self.manager.exists(license_version):
                critical_error_message_box(
                    translate('BiblesPlugin.ImportWizardForm', 'Bible Exists'),
                    translate('BiblesPlugin.ImportWizardForm',
                    'This Bible already exists. Please import '
                    'a different Bible or first delete the existing one.'))
                self.versionNameEdit.setFocus()
                return False
            elif os.path.exists(os.path.join(path, clean_filename(
                license_version))):
                critical_error_message_box(
                    translate('BiblesPlugin.ImportWizardForm', 'Bible Exists'),
                    translate('BiblesPlugin.ImportWizardForm',
                    'This Bible already exists. Please import '
                    'a different Bible or first delete the existing one.'))
                self.versionNameEdit.setFocus()
                return False
            return True
        if self.currentPage() == self.progressPage:
            return True

    def onWebSourceComboBoxIndexChanged(self, index):
        """
        Setup the list of Bibles when you select a different source on the web
        download page.

        ``index``
            The index of the combo box.
        """
        self.webTranslationComboBox.clear()
        bibles = self.web_bible_list[index].keys()
        bibles.sort(cmp=locale_compare)
        self.webTranslationComboBox.addItems(bibles)

    def onOsisBrowseButtonClicked(self):
        """
        Show the file open dialog for the OSIS file.
        """
        self.getFileName(WizardStrings.OpenTypeFile % WizardStrings.OSIS,
            self.osisFileEdit)

    def onCsvBooksBrowseButtonClicked(self):
        """
        Show the file open dialog for the books CSV file.
        """
        self.getFileName(WizardStrings.OpenTypeFile % WizardStrings.CSV,
            self.csvBooksEdit, u'%s (*.csv)'
            % translate('BiblesPlugin.ImportWizardForm', 'CSV File'))

    def onCsvVersesBrowseButtonClicked(self):
        """
        Show the file open dialog for the verses CSV file.
        """
        self.getFileName(WizardStrings.OpenTypeFile % WizardStrings.CSV,
            self.csvVersesEdit, u'%s (*.csv)'
            % translate('BiblesPlugin.ImportWizardForm', 'CSV File'))

    def onOpenSongBrowseButtonClicked(self):
        """
        Show the file open dialog for the OpenSong file.
        """
        self.getFileName(WizardStrings.OpenTypeFile % WizardStrings.OS,
            self.openSongFileEdit)

    def onOpenlp1BrowseButtonClicked(self):
        """
        Show the file open dialog for the openlp.org 1.x file.
        """
        self.getFileName(WizardStrings.OpenTypeFile % UiStrings().OLPV1,
            self.openlp1FileEdit, u'%s (*.bible)' %
            translate('BiblesPlugin.ImportWizardForm',
            'openlp.org 1.x Bible Files'))

    def registerFields(self):
        """
        Register the bible import wizard fields.
        """
        self.selectPage.registerField(u'source_format', self.formatComboBox)
        self.selectPage.registerField(u'osis_location', self.osisFileEdit)
        self.selectPage.registerField(u'csv_booksfile', self.csvBooksEdit)
        self.selectPage.registerField(u'csv_versefile', self.csvVersesEdit)
        self.selectPage.registerField(u'opensong_file', self.openSongFileEdit)
        self.selectPage.registerField(u'web_location', self.webSourceComboBox)
        self.selectPage.registerField(
            u'web_biblename', self.webTranslationComboBox)
        self.selectPage.registerField(u'proxy_server', self.webServerEdit)
        self.selectPage.registerField(u'proxy_username', self.webUserEdit)
        self.selectPage.registerField(u'proxy_password', self.webPasswordEdit)
        self.selectPage.registerField(u'openlp1_location', self.openlp1FileEdit)
        self.licenseDetailsPage.registerField(
            u'license_version', self.versionNameEdit)
        self.licenseDetailsPage.registerField(
            u'license_copyright', self.copyrightEdit)
        self.licenseDetailsPage.registerField(
            u'license_permissions', self.permissionsEdit)

    def setDefaults(self):
        """
        Set default values for the wizard pages.
        """
        settings = Settings()
        settings.beginGroup(self.plugin.settingsSection)
        self.restart()
        self.finishButton.setVisible(False)
        self.cancelButton.setVisible(True)
        self.setField(u'source_format', QtCore.QVariant(0))
        self.setField(u'osis_location', QtCore.QVariant(''))
        self.setField(u'csv_booksfile', QtCore.QVariant(''))
        self.setField(u'csv_versefile', QtCore.QVariant(''))
        self.setField(u'opensong_file', QtCore.QVariant(''))
        self.setField(u'web_location', QtCore.QVariant(WebDownload.Crosswalk))
        self.setField(u'web_biblename',
            QtCore.QVariant(self.webTranslationComboBox.currentIndex()))
        self.setField(u'proxy_server',
            settings.value(u'proxy address', QtCore.QVariant(u'')))
        self.setField(u'proxy_username',
            settings.value(u'proxy username', QtCore.QVariant(u'')))
        self.setField(u'proxy_password',
            settings.value(u'proxy password', QtCore.QVariant(u'')))
        self.setField(u'openlp1_location', QtCore.QVariant(''))
        self.setField(u'license_version',
            QtCore.QVariant(self.versionNameEdit.text()))
        self.setField(u'license_copyright',
            QtCore.QVariant(self.copyrightEdit.text()))
        self.setField(u'license_permissions',
            QtCore.QVariant(self.permissionsEdit.text()))
        self.onWebSourceComboBoxIndexChanged(WebDownload.Crosswalk)
        settings.endGroup()

    def loadWebBibles(self):
        """
        Load the lists of Crosswalk, BibleGateway and Bibleserver bibles.
        """
        # Load Crosswalk Bibles.
        self.loadBibleResource(WebDownload.Crosswalk)
        # Load BibleGateway Bibles.
        self.loadBibleResource(WebDownload.BibleGateway)
        # Load and Bibleserver Bibles.
        self.loadBibleResource(WebDownload.Bibleserver)

    def loadBibleResource(self, download_type):
        """
        Loads a web bible from bible_resources.sqlite.

        ``download_type``
            The WebDownload type e.g. bibleserver.
        """
        self.web_bible_list[download_type] = {}
        bibles = BiblesResourcesDB.get_webbibles(
            WebDownload.Names[download_type])
        for bible in bibles:
            version = bible[u'name']
            name = bible[u'abbreviation']
            self.web_bible_list[download_type][version] = name.strip()

    def preWizard(self):
        """
        Prepare the UI for the import.
        """
        OpenLPWizard.preWizard(self)
        bible_type = self.field(u'source_format').toInt()[0]
        if bible_type == BibleFormat.WebDownload:
            self.progressLabel.setText(translate(
                'BiblesPlugin.ImportWizardForm',
                'Registering Bible...'))
        else:
            self.progressLabel.setText(WizardStrings.StartingImport)
        Receiver.send_message(u'openlp_process_events')

    def performWizard(self):
        """
        Perform the actual import.
        """
        bible_type = self.field(u'source_format').toInt()[0]
        license_version = unicode(self.field(u'license_version').toString())
        license_copyright = unicode(self.field(u'license_copyright').toString())
        license_permissions = \
            unicode(self.field(u'license_permissions').toString())
        importer = None
        if bible_type == BibleFormat.OSIS:
            # Import an OSIS bible.
            importer = self.manager.import_bible(BibleFormat.OSIS,
                name=license_version,
                filename=unicode(self.field(u'osis_location').toString())
            )
        elif bible_type == BibleFormat.CSV:
            # Import a CSV bible.
            importer = self.manager.import_bible(BibleFormat.CSV,
                name=license_version,
                booksfile=unicode(self.field(u'csv_booksfile').toString()),
                versefile=unicode(self.field(u'csv_versefile').toString())
            )
        elif bible_type == BibleFormat.OpenSong:
            # Import an OpenSong bible.
            importer = self.manager.import_bible(BibleFormat.OpenSong,
                name=license_version,
                filename=unicode(self.field(u'opensong_file').toString())
            )
        elif bible_type == BibleFormat.WebDownload:
            # Import a bible from the web.
            self.progressBar.setMaximum(1)
            download_location = self.field(u'web_location').toInt()[0]
            bible_version = unicode(self.webTranslationComboBox.currentText())
            bible = self.web_bible_list[download_location][bible_version]
            importer = self.manager.import_bible(
                BibleFormat.WebDownload, name=license_version,
                download_source=WebDownload.Names[download_location],
                download_name=bible,
                proxy_server=unicode(self.field(u'proxy_server').toString()),
                proxy_username=\
                    unicode(self.field(u'proxy_username').toString()),
                proxy_password=unicode(self.field(u'proxy_password').toString())
            )
        elif bible_type == BibleFormat.OpenLP1:
            # Import an openlp.org 1.x bible.
            importer = self.manager.import_bible(BibleFormat.OpenLP1,
                name=license_version,
                filename=unicode(self.field(u'openlp1_location').toString())
            )
        if importer.do_import(license_version):
            self.manager.save_meta_data(license_version, license_version,
                license_copyright, license_permissions)
            self.manager.reload_bibles()
            if bible_type == BibleFormat.WebDownload:
                self.progressLabel.setText(
                    translate('BiblesPlugin.ImportWizardForm', 'Registered '
                    'Bible. Please note, that verses will be downloaded on\n'
                    'demand and thus an internet connection is required.'))
            else:
                self.progressLabel.setText(WizardStrings.FinishedImport)
        else:
            self.progressLabel.setText(translate(
                'BiblesPlugin.ImportWizardForm', 'Your Bible import failed.'))
            del self.manager.db_cache[importer.name]
            delete_database(self.plugin.settingsSection, importer.file)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
import cgi
import datetime
import os

from PyQt4 import QtCore, QtGui
from lxml import html

from openlp.core.lib import translate, get_text_file_string, Receiver
from openlp.core.lib.ui import UiStrings
from openlp.core.lib.settings import Settings
from openlp.core.ui.printservicedialog import Ui_PrintServiceDialog, ZoomSize
from openlp.core.utils import AppLocation

DEFAULT_CSS = """/*
Edit this file to customize the service order print. Note, that not all CSS
properties are supported. See:
http://doc.trolltech.com/4.7/richtext-html-subset.html#css-properties
*/

.serviceTitle {
   font-weight: 600;
   font-size: x-large;
   color: black;
}

.item {
   color: black;
}

.itemTitle {
   font-weight: 600;
   font-size: large;
}

.itemText {
   margin-top: 10px;
}

.itemFooter {
   font-size: 8px;
}

.itemNotes {}

.itemNotesTitle {
   font-weight: bold;
   font-size: 12px;
}

.itemNotesText {
   font-size: 11px;
}

.media {}

.mediaTitle {
    font-weight: bold;
    font-size: 11px;
}

.mediaText {}

.imageList {}

.customNotes {
   margin-top: 10px;
}

.customNotesTitle {
   font-weight: bold;
   font-size: 11px;
}

.customNotesText {
   font-size: 11px;
}

.newPage {
    page-break-before: always;
}
"""

class PrintServiceForm(QtGui.QDialog, Ui_PrintServiceDialog):

    def __init__(self, mainWindow, serviceManager):
        """
        Constructor
        """
        QtGui.QDialog.__init__(self, mainWindow)
        self.mainWindow = mainWindow
        self.serviceManager = serviceManager
        self.printer = QtGui.QPrinter()
        self.printDialog = QtGui.QPrintDialog(self.printer, self)
        self.document = QtGui.QTextDocument()
        self.zoom = 0
        self.setupUi(self)
        # Load the settings for the dialog.
        settings = Settings()
        settings.beginGroup(u'advanced')
        self.slideTextCheckBox.setChecked(settings.value(
            u'print slide text', QtCore.QVariant(False)).toBool())
        self.pageBreakAfterText.setChecked(settings.value(
            u'add page break', QtCore.QVariant(False)).toBool())
        if not self.slideTextCheckBox.isChecked():
            self.pageBreakAfterText.setDisabled(True)
        self.metaDataCheckBox.setChecked(settings.value(
            u'print file meta data', QtCore.QVariant(False)).toBool())
        self.notesCheckBox.setChecked(settings.value(
            u'print notes', QtCore.QVariant(False)).toBool())
        self.zoomComboBox.setCurrentIndex(settings.value(
            u'display size', QtCore.QVariant(0)).toInt()[0])
        settings.endGroup()
        # Signals
        QtCore.QObject.connect(self.printButton,
            QtCore.SIGNAL(u'triggered()'), self.printServiceOrder)
        QtCore.QObject.connect(self.zoomOutButton,
            QtCore.SIGNAL(u'clicked()'), self.zoomOut)
        QtCore.QObject.connect(self.zoomInButton,
            QtCore.SIGNAL(u'clicked()'), self.zoomIn)
        QtCore.QObject.connect(self.zoomOriginalButton,
            QtCore.SIGNAL(u'clicked()'), self.zoomOriginal)
        QtCore.QObject.connect(self.previewWidget,
            QtCore.SIGNAL(u'paintRequested(QPrinter *)'), self.paintRequested)
        QtCore.QObject.connect(self.zoomComboBox,
            QtCore.SIGNAL(u'currentIndexChanged(int)'), self.displaySizeChanged)
        QtCore.QObject.connect(self.plainCopy,
            QtCore.SIGNAL(u'triggered()'), self.copyText)
        QtCore.QObject.connect(self.htmlCopy,
            QtCore.SIGNAL(u'triggered()'), self.copyHtmlText)
        QtCore.QObject.connect(self.slideTextCheckBox,
            QtCore.SIGNAL(u'stateChanged(int)'),
            self.onSlideTextCheckBoxChanged)
        self.updatePreviewText()

    def toggleOptions(self, checked):
        self.optionsWidget.setVisible(checked)
        if checked:
            left = self.optionsButton.pos().x()
            top = self.toolbar.height()
            self.optionsWidget.move(left, top)
            self.titleLineEdit.setFocus()
        else:
            self.saveOptions()
        self.updatePreviewText()

    def updatePreviewText(self):
        """
        Creates the html text and updates the html of *self.document*.
        """
        html_data = self._addElement(u'html')
        self._addElement(u'head', parent=html_data)
        self._addElement(u'title', unicode(self.titleLineEdit.text()),
            html_data.head)
        css_path = os.path.join(
            AppLocation.get_data_path(), u'service_print.css')
        custom_css = get_text_file_string(css_path)
        if not custom_css:
            custom_css = DEFAULT_CSS
        self._addElement(u'style', custom_css, html_data.head,
            attribute=(u'type', u'text/css'))
        self._addElement(u'body', parent=html_data)
        self._addElement(u'h1', cgi.escape(unicode(self.titleLineEdit.text())),
            html_data.body, classId=u'serviceTitle')
        for index, item in enumerate(self.serviceManager.serviceItems):
            self._addPreviewItem(html_data.body, item[u'service_item'], index)
        # Add the custom service notes:
        if self.footerTextEdit.toPlainText():
            div = self._addElement(u'div', parent=html_data.body,
                classId=u'customNotes')
            self._addElement(u'span', translate('OpenLP.ServiceManager',
                'Custom Service Notes: '), div, classId=u'customNotesTitle')
            self._addElement(u'span',
                cgi.escape(self.footerTextEdit.toPlainText()),
                div, classId=u'customNotesText')
        self.document.setHtml(html.tostring(html_data))
        self.previewWidget.updatePreview()

    def _addPreviewItem(self, body, item, index):
        div = self._addElement(u'div', classId=u'item', parent=body)
        # Add the title of the service item.
        item_title = self._addElement(u'h2', parent=div, classId=u'itemTitle')
        self._addElement(u'img', parent=item_title,
            attribute=(u'src', item.icon))
        self._addElement(u'span',
            u'&nbsp;' + cgi.escape(item.get_display_title()), item_title)
        if self.slideTextCheckBox.isChecked():
            # Add the text of the service item.
            if item.is_text():
                verse_def = None
                for slide in item.get_frames():
                    if not verse_def or verse_def != slide[u'verseTag']:
                        text_div = self._addElement(u'div', parent=div,
                            classId=u'itemText')
                    else:
                        self._addElement(u'br', parent=text_div)
                    self._addElement(u'span', slide[u'html'], text_div)
                    verse_def = slide[u'verseTag']
                # Break the page before the div element.
                if index != 0 and self.pageBreakAfterText.isChecked():
                    div.set(u'class', u'item newPage')
            # Add the image names of the service item.
            elif item.is_image():
                ol = self._addElement(u'ol', parent=div, classId=u'imageList')
                for slide in range(len(item.get_frames())):
                    self._addElement(u'li', item.get_frame_title(slide), ol)
            # add footer
            foot_text = item.foot_text
            foot_text = foot_text.partition(u'<br>')[2]
            if foot_text:
                foot_text = cgi.escape(foot_text.replace(u'<br>', u'\n'))
                self._addElement(u'div', foot_text.replace(u'\n', u'<br>'),
                    parent=div, classId=u'itemFooter')
        # Add service items' notes.
        if self.notesCheckBox.isChecked():
            if item.notes:
                p = self._addElement(u'div', classId=u'itemNotes', parent=div)
                self._addElement(u'span',
                    translate('OpenLP.ServiceManager', 'Notes: '), p,
                    classId=u'itemNotesTitle')
                self._addElement(u'span',
                    cgi.escape(unicode(item.notes)).replace(u'\n', u'<br>'), p,
                    classId=u'itemNotesText')
        # Add play length of media files.
        if item.is_media() and self.metaDataCheckBox.isChecked():
            tme = item.media_length
            if item.end_time > 0:
                tme = item.end_time - item.start_time
            title = self._addElement(u'div', classId=u'media', parent=div)
            self._addElement(u'span', translate('OpenLP.ServiceManager',
                'Playing time: '), title, classId=u'mediaTitle')
            self._addElement(u'span', unicode(datetime.timedelta(seconds=tme)),
                title, classId=u'mediaText')

    def _addElement(self, tag, text=None, parent=None, classId=None,
        attribute=None):
        """
        Creates a html element. If ``text`` is given, the element's text will
        set and if a ``parent`` is given, the element is appended.

        ``tag``
            The html tag, e. g. ``u'span'``. Defaults to ``None``.

        ``text``
            The text for the tag. Defaults to ``None``.

        ``parent``
            The parent element. Defaults to ``None``.

        ``classId``
            Value for the class attribute

        ``attribute``
            Tuple name/value pair to add as an optional attribute
        """
        if text is not None:
            element = html.fragment_fromstring(unicode(text), create_parent=tag)
        else:
            element = html.Element(tag)
        if parent is not None:
            parent.append(element)
        if classId is not None:
            element.set(u'class', classId)
        if attribute is not None:
            element.set(attribute[0], attribute[1])
        return element

    def paintRequested(self, printer):
        """
        Paint the preview of the *self.document*.

        ``printer``
            A *QPrinter* object.
        """
        self.document.print_(printer)

    def displaySizeChanged(self, display):
        """
        The Zoom Combo box has changed so set up the size.
        """
        if display == ZoomSize.Page:
            self.previewWidget.fitInView()
        elif display == ZoomSize.Width:
            self.previewWidget.fitToWidth()
        elif display == ZoomSize.OneHundred:
            self.previewWidget.fitToWidth()
            self.previewWidget.zoomIn(1)
        elif display == ZoomSize.SeventyFive:
            self.previewWidget.fitToWidth()
            self.previewWidget.zoomIn(0.75)
        elif display == ZoomSize.Fifty:
            self.previewWidget.fitToWidth()
            self.previewWidget.zoomIn(0.5)
        elif display == ZoomSize.TwentyFive:
            self.previewWidget.fitToWidth()
            self.previewWidget.zoomIn(0.25)
        settings = Settings()
        settings.beginGroup(u'advanced')
        settings.setValue(u'display size', QtCore.QVariant(display))
        settings.endGroup()

    def copyText(self):
        """
        Copies the display text to the clipboard as plain text
        """
        self.update_song_usage()
        cursor = QtGui.QTextCursor(self.document)
        cursor.select(QtGui.QTextCursor.Document)
        clipboard_text = cursor.selectedText()
        # We now have the unprocessed unicode service text in the cursor
        # So we replace u2028 with \n and u2029 with \n\n and a few others
        clipboard_text = clipboard_text.replace(u'\u2028', u'\n')
        clipboard_text = clipboard_text.replace(u'\u2029', u'\n\n')
        clipboard_text = clipboard_text.replace(u'\u2018', u'\'')
        clipboard_text = clipboard_text.replace(u'\u2019', u'\'')
        clipboard_text = clipboard_text.replace(u'\u201c', u'"')
        clipboard_text = clipboard_text.replace(u'\u201d', u'"')
        clipboard_text = clipboard_text.replace(u'\u2026', u'...')
        clipboard_text = clipboard_text.replace(u'\u2013', u'-')
        clipboard_text = clipboard_text.replace(u'\u2014', u'-')
        # remove the icon from the text
        clipboard_text = clipboard_text.replace(u'\ufffc\xa0', u'')
        # and put it all on the clipboard
        self.mainWindow.clipboard.setText(clipboard_text)

    def copyHtmlText(self):
        """
        Copies the display text to the clipboard as Html
        """
        self.update_song_usage()
        self.mainWindow.clipboard.setText(self.document.toHtml())

    def printServiceOrder(self):
        """
        Called, when the *printButton* is clicked. Opens the *printDialog*.
        """
        if not self.printDialog.exec_():
            return
        self.update_song_usage()
        # Print the document.
        self.document.print_(self.printer)

    def zoomIn(self):
        """
        Called when *zoomInButton* is clicked.
        """
        self.previewWidget.zoomIn()
        self.zoom -= 0.1

    def zoomOut(self):
        """
        Called when *zoomOutButton* is clicked.
        """
        self.previewWidget.zoomOut()
        self.zoom += 0.1

    def zoomOriginal(self):
        """
        Called when *zoomOutButton* is clicked.
        """
        self.previewWidget.zoomIn(1 + self.zoom)
        self.zoom = 0

    def updateTextFormat(self, value):
        """
        Called when html copy check box is selected.
        """
        if value == QtCore.Qt.Checked:
            self.copyTextButton.setText(UiStrings().CopyToHtml)
        else:
            self.copyTextButton.setText(UiStrings().CopyToText)

    def onSlideTextCheckBoxChanged(self, state):
        """
        Disable or enable the ``pageBreakAfterText`` checkbox  as it should only
        be enabled, when the ``slideTextCheckBox`` is enabled.
        """
        self.pageBreakAfterText.setDisabled(state == QtCore.Qt.Unchecked)

    def saveOptions(self):
        """
        Save the settings and close the dialog.
        """
        # Save the settings for this dialog.
        settings = Settings()
        settings.beginGroup(u'advanced')
        settings.setValue(u'print slide text',
            QtCore.QVariant(self.slideTextCheckBox.isChecked()))
        settings.setValue(u'add page break',
            QtCore.QVariant(self.pageBreakAfterText.isChecked()))
        settings.setValue(u'print file meta data',
            QtCore.QVariant(self.metaDataCheckBox.isChecked()))
        settings.setValue(u'print notes',
            QtCore.QVariant(self.notesCheckBox.isChecked()))
        settings.endGroup()

    def update_song_usage(self):
        # Only continue when we include the song's text.
        if not self.slideTextCheckBox.isChecked():
            return
        for item in self.serviceManager.serviceItems:
            # Trigger Audit requests
            Receiver.send_message(u'print_service_started',
                [item[u'service_item']])



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtGui

class SettingsTab(QtGui.QWidget):
    """
    SettingsTab is a helper widget for plugins to define Tabs for the settings
    dialog.
    """
    def __init__(self, parent, title, visible_title=None, icon_path=None):
        """
        Constructor to create the Settings tab item.

        ``title``
            The title of the tab, which is used internally for the tab handling.

        ``visible_title``
            The title of the tab, which is usually displayed on the tab.
        """
        QtGui.QWidget.__init__(self, parent)
        self.tabTitle = title
        self.tabTitleVisible = visible_title
        self.settingsSection = self.tabTitle.lower()
        if icon_path:
            self.iconPath = icon_path
        self.setupUi()
        self.retranslateUi()
        self.initialise()
        self.load()

    def setupUi(self):
        """
        Setup the tab's interface.
        """
        self.tabLayout = QtGui.QHBoxLayout(self)
        self.tabLayout.setObjectName(u'tabLayout')
        self.leftColumn = QtGui.QWidget(self)
        self.leftColumn.setObjectName(u'leftColumn')
        self.leftLayout = QtGui.QVBoxLayout(self.leftColumn)
        self.leftLayout.setMargin(0)
        self.leftLayout.setObjectName(u'leftLayout')
        self.tabLayout.addWidget(self.leftColumn)
        self.rightColumn = QtGui.QWidget(self)
        self.rightColumn.setObjectName(u'rightColumn')
        self.rightLayout = QtGui.QVBoxLayout(self.rightColumn)
        self.rightLayout.setMargin(0)
        self.rightLayout.setObjectName(u'rightLayout')
        self.tabLayout.addWidget(self.rightColumn)

    def resizeEvent(self, event=None):
        """
        Resize the sides in two equal halves if the layout allows this.
        """
        if event:
            QtGui.QWidget.resizeEvent(self, event)
        width = self.width() - self.tabLayout.spacing() - \
            self.tabLayout.contentsMargins().left() - \
            self.tabLayout.contentsMargins().right()
        left_width = min(width - self.rightColumn.minimumSizeHint().width(),
            width / 2)
        left_width = max(left_width, self.leftColumn.minimumSizeHint().width())
        self.leftColumn.setFixedWidth(left_width)

    def retranslateUi(self):
        """
        Setup the interface translation strings.
        """
        pass

    def initialise(self):
        """
        Do any extra initialisation here.
        """
        pass

    def load(self):
        """
        Load settings from disk.
        """
        pass

    def save(self):
        """
        Save settings to disk.
        """
        pass

    def cancel(self):
        """
        Reset any settings if cancel triggered
        """
        self.load()

    def postSetUp(self, postUpdate=False):
        """
        Changes which need to be made after setup of application

        ``postUpdate``
            Indicates if called before or after updates.

        """
        pass

    def tabVisible(self):
        """
        Tab has just been made visible to the user
        """
        pass



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from aboutdialog import Ui_AboutDialog
from openlp.core.lib import translate
from openlp.core.utils import get_application_version

class AboutForm(QtGui.QDialog, Ui_AboutDialog):
    """
    The About dialog
    """

    def __init__(self, parent):
        """
        Do some initialisation stuff
        """
        QtGui.QDialog.__init__(self, parent)
        applicationVersion = get_application_version()
        self.setupUi(self)
        about_text = self.aboutTextEdit.toPlainText()
        about_text = about_text.replace(u'<version>',
            applicationVersion[u'version'])
        if applicationVersion[u'build']:
            build_text = unicode(translate('OpenLP.AboutForm', ' build %s')) % \
                applicationVersion[u'build']
        else:
            build_text = u''
        about_text = about_text.replace(u'<revision>', build_text)
        self.aboutTextEdit.setPlainText(about_text)
        QtCore.QObject.connect(self.volunteerButton,
            QtCore.SIGNAL(u'clicked()'), self.onVolunteerButtonClicked)

    def onVolunteerButtonClicked(self):
        """
        Launch a web browser and go to the contribute page on the site.
        """
        import webbrowser
        url = u'http://openlp.org/en/contribute'
        webbrowser.open_new(url)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging

from PyQt4 import QtCore

from openlp.core.lib import Plugin, StringContent, build_icon, translate
from openlp.core.lib.db import Manager
from openlp.core.lib.ui import create_action, UiStrings
from openlp.core.lib.settings import Settings
from openlp.core.lib.theme import VerticalType
from openlp.core.utils.actions import ActionList
from openlp.plugins.alerts.lib import AlertsManager, AlertsTab
from openlp.plugins.alerts.lib.db import init_schema
from openlp.plugins.alerts.forms import AlertForm

log = logging.getLogger(__name__)

JAVASCRIPT = """
    function show_alert(alerttext, position){
        var text = document.getElementById('alert');
        text.innerHTML = alerttext;
        if(alerttext == '') {
            text.style.visibility = 'hidden';
            return 0;
        }
        if(position == ''){
            position = getComputedStyle(text, '').verticalAlign;
        }
        switch(position)
        {
            case 'top':
                text.style.top = '0px';
                break;
            case 'middle':
                text.style.top = ((window.innerHeight - text.clientHeight) / 2)
                    + 'px';
                break;
            case 'bottom':
                text.style.top = (window.innerHeight - text.clientHeight)
                    + 'px';
                break;
        }
        text.style.visibility = 'visible';
        return text.clientHeight;
    }

    function update_css(align, font, size, color, bgcolor){
        var text = document.getElementById('alert');
        text.style.fontSize = size + "pt";
        text.style.fontFamily = font;
        text.style.color = color;
        text.style.backgroundColor = bgcolor;
        switch(align)
        {
            case 'top':
                text.style.top = '0px';
                break;
            case 'middle':
                text.style.top = ((window.innerHeight - text.clientHeight) / 2)
                    + 'px';
                break;
            case 'bottom':
                text.style.top = (window.innerHeight - text.clientHeight)
                    + 'px';
                break;
        }
    }
"""
CSS = """
    #alert {
        position: absolute;
        left: 0px;
        top: 0px;
        z-index: 10;
        width: 100%%;
        vertical-align: %s;
        font-family: %s;
        font-size: %spt;
        color: %s;
        background-color: %s;
        word-wrap: break-word;
    }
"""

HTML = """
    <div id="alert" style="visibility:hidden"></div>
"""

class AlertsPlugin(Plugin):
    log.info(u'Alerts Plugin loaded')

    def __init__(self, plugin_helpers):
        Plugin.__init__(self, u'alerts', plugin_helpers,
            settings_tab_class=AlertsTab)
        self.weight = -3
        self.iconPath = u':/plugins/plugin_alerts.png'
        self.icon = build_icon(self.iconPath)
        self.alertsmanager = AlertsManager(self)
        self.manager = Manager(u'alerts', init_schema)
        self.alertForm = AlertForm(self)

    def addToolsMenuItem(self, tools_menu):
        """
        Give the alerts plugin the opportunity to add items to the
        **Tools** menu.

        ``tools_menu``
            The actual **Tools** menu item, so that your actions can
            use it as their parent.
        """
        log.info(u'add tools menu')
        self.toolsAlertItem = create_action(tools_menu, u'toolsAlertItem',
            text=translate('AlertsPlugin', '&Alert'),
            icon=u':/plugins/plugin_alerts.png',
            statustip=translate('AlertsPlugin', 'Show an alert message.'),
            visible=False, shortcuts=[u'F7'], triggers=self.onAlertsTrigger)
        self.serviceManager.mainwindow.toolsMenu.addAction(self.toolsAlertItem)

    def initialise(self):
        log.info(u'Alerts Initialising')
        Plugin.initialise(self)
        self.toolsAlertItem.setVisible(True)
        action_list = ActionList.get_instance()
        action_list.add_action(self.toolsAlertItem, unicode(UiStrings().Tools))

    def finalise(self):
        """
        Tidy up on exit
        """
        log.info(u'Alerts Finalising')
        self.manager.finalise()
        Plugin.finalise(self)
        self.toolsAlertItem.setVisible(False)
        action_list = ActionList.get_instance()
        action_list.remove_action(self.toolsAlertItem, u'Tools')

    def toggleAlertsState(self):
        self.alertsActive = not self.alertsActive
        Settings().setValue(self.settingsSection + u'/active',
            QtCore.QVariant(self.alertsActive))

    def onAlertsTrigger(self):
        self.alertForm.loadList()
        self.alertForm.exec_()

    def about(self):
        about_text = translate('AlertsPlugin', '<strong>Alerts Plugin</strong>'
            '<br />The alert plugin controls the displaying of nursery alerts '
            'on the display screen.')
        return about_text

    def setPluginTextStrings(self):
        """
        Called to define all translatable texts of the plugin
        """
        ## Name PluginList ##
        self.textStrings[StringContent.Name] = {
            u'singular': translate('AlertsPlugin', 'Alert', 'name singular'),
            u'plural': translate('AlertsPlugin', 'Alerts', 'name plural')
        }
        ## Name for MediaDockManager, SettingsManager ##
        self.textStrings[StringContent.VisibleName] = {
            u'title': translate('AlertsPlugin', 'Alerts', 'container title')
        }

    def getDisplayJavaScript(self):
        """
        Add Javascript to the main display.
        """
        return JAVASCRIPT

    def getDisplayCss(self):
        """
        Add CSS to the main display.
        """
        align = VerticalType.Names[self.settingsTab.location]
        return CSS % (align, self.settingsTab.font_face,
            self.settingsTab.font_size, self.settingsTab.font_color,
            self.settingsTab.bg_color)

    def getDisplayHtml(self):
        """
        Add HTML to the main display.
        """
        return HTML

    def refreshCss(self, frame):
        """
        Trigger an update of the CSS in the maindisplay.

        ``frame``
            The Web frame holding the page.
        """
        align = VerticalType.Names[self.settingsTab.location]
        frame.evaluateJavaScript(u'update_css("%s", "%s", "%s", "%s", "%s")' %
            (align, self.settingsTab.font_face, self.settingsTab.font_size,
            self.settingsTab.font_color, self.settingsTab.bg_color))



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
import re

from PyQt4 import QtGui, QtCore

from openlp.core.lib import translate
from openlp.core.utils import CONTROL_CHARS, locale_direct_compare
from db import Author
from ui import SongStrings

WHITESPACE = re.compile(r'[\W_]+', re.UNICODE)
APOSTROPHE = re.compile(u'[\'`’ʻ′]', re.UNICODE)
PATTERN = re.compile(r"\\([a-z]{1,32})(-?\d{1,10})?[ ]?|\\'"
    r"([0-9a-f]{2})|\\([^a-z])|([{}])|[\r\n]+|(.)", re.I)
# RTF control words which specify a "destination" to be ignored.
DESTINATIONS = frozenset((
    u'aftncn', u'aftnsep', u'aftnsepc', u'annotation', u'atnauthor',
    u'atndate', u'atnicn', u'atnid', u'atnparent', u'atnref', u'atntime',
    u'atrfend', u'atrfstart', u'author', u'background', u'bkmkend',
    u'bkmkstart', u'blipuid', u'buptim', u'category',
    u'colorschememapping', u'colortbl', u'comment', u'company', u'creatim',
    u'datafield', u'datastore', u'defchp', u'defpap', u'do', u'doccomm',
    u'docvar', u'dptxbxtext', u'ebcend', u'ebcstart', u'factoidname',
    u'falt', u'fchars', u'ffdeftext', u'ffentrymcr', u'ffexitmcr',
    u'ffformat', u'ffhelptext', u'ffl', u'ffname', u'ffstattext', u'field',
    u'file', u'filetbl', u'fldinst', u'fldrslt', u'fldtype', u'fname',
    u'fontemb', u'fontfile', u'footer', u'footerf', u'footerl', u'footerr',
    u'footnote', u'formfield', u'ftncn', u'ftnsep', u'ftnsepc', u'g',
    u'generator', u'gridtbl', u'header', u'headerf', u'headerl',
    u'headerr', u'hl', u'hlfr', u'hlinkbase', u'hlloc', u'hlsrc', u'hsv',
    u'htmltag', u'info', u'keycode', u'keywords', u'latentstyles',
    u'lchars', u'levelnumbers', u'leveltext', u'lfolevel', u'linkval',
    u'list', u'listlevel', u'listname', u'listoverride',
    u'listoverridetable', u'listpicture', u'liststylename', u'listtable',
    u'listtext', u'lsdlockedexcept', u'macc', u'maccPr', u'mailmerge',
    u'maln', u'malnScr', u'manager', u'margPr', u'mbar', u'mbarPr',
    u'mbaseJc', u'mbegChr', u'mborderBox', u'mborderBoxPr', u'mbox',
    u'mboxPr', u'mchr', u'mcount', u'mctrlPr', u'md', u'mdeg', u'mdegHide',
    u'mden', u'mdiff', u'mdPr', u'me', u'mendChr', u'meqArr', u'meqArrPr',
    u'mf', u'mfName', u'mfPr', u'mfunc', u'mfuncPr', u'mgroupChr',
    u'mgroupChrPr', u'mgrow', u'mhideBot', u'mhideLeft', u'mhideRight',
    u'mhideTop', u'mhtmltag', u'mlim', u'mlimloc', u'mlimlow',
    u'mlimlowPr', u'mlimupp', u'mlimuppPr', u'mm', u'mmaddfieldname',
    u'mmath', u'mmathPict', u'mmathPr', u'mmaxdist', u'mmc', u'mmcJc',
    u'mmconnectstr', u'mmconnectstrdata', u'mmcPr', u'mmcs',
    u'mmdatasource', u'mmheadersource', u'mmmailsubject', u'mmodso',
    u'mmodsofilter', u'mmodsofldmpdata', u'mmodsomappedname',
    u'mmodsoname', u'mmodsorecipdata', u'mmodsosort', u'mmodsosrc',
    u'mmodsotable', u'mmodsoudl', u'mmodsoudldata', u'mmodsouniquetag',
    u'mmPr', u'mmquery', u'mmr', u'mnary', u'mnaryPr', u'mnoBreak',
    u'mnum', u'mobjDist', u'moMath', u'moMathPara', u'moMathParaPr',
    u'mopEmu', u'mphant', u'mphantPr', u'mplcHide', u'mpos', u'mr',
    u'mrad', u'mradPr', u'mrPr', u'msepChr', u'mshow', u'mshp', u'msPre',
    u'msPrePr', u'msSub', u'msSubPr', u'msSubSup', u'msSubSupPr', u'msSup',
    u'msSupPr', u'mstrikeBLTR', u'mstrikeH', u'mstrikeTLBR', u'mstrikeV',
    u'msub', u'msubHide', u'msup', u'msupHide', u'mtransp', u'mtype',
    u'mvertJc', u'mvfmf', u'mvfml', u'mvtof', u'mvtol', u'mzeroAsc',
    u'mzFrodesc', u'mzeroWid', u'nesttableprops', u'nextfile',
    u'nonesttables', u'objalias', u'objclass', u'objdata', u'object',
    u'objname', u'objsect', u'objtime', u'oldcprops', u'oldpprops',
    u'oldsprops', u'oldtprops', u'oleclsid', u'operator', u'panose',
    u'password', u'passwordhash', u'pgp', u'pgptbl', u'picprop', u'pict',
    u'pn', u'pnseclvl', u'pntext', u'pntxta', u'pntxtb', u'printim',
    u'private', u'propname', u'protend', u'protstart', u'protusertbl',
    u'pxe', u'result', u'revtbl', u'revtim', u'rsidtbl', u'rxe', u'shp',
    u'shpgrp', u'shpinst', u'shppict', u'shprslt', u'shptxt', u'sn', u'sp',
    u'staticval', u'stylesheet', u'subject', u'sv', u'svb', u'tc',
    u'template', u'themedata', u'title', u'txe', u'ud', u'upr',
    u'userprops', u'wgrffmtfilter', u'windowcaption', u'writereservation',
    u'writereservhash', u'xe', u'xform', u'xmlattrname', u'xmlattrvalue',
    u'xmlclose', u'xmlname', u'xmlnstbl', u'xmlopen'))
# Translation of some special characters.
SPECIAL_CHARS = {
    u'par': u'\n',
    u'sect': u'\n\n',
    # Required page and column break.
    # Would be good if we could split verse into subverses here.
    u'page': u'\n\n',
    u'column': u'\n\n',
    # Soft breaks.
    u'softpage': u'[---]',
    u'softcol': u'[---]',
    u'line': u'\n',
    u'tab': u'\t',
    u'emdash': u'\u2014',
    u'endash': u'\u2013',
    u'emspace': u'\u2003',
    u'enspace': u'\u2002',
    u'qmspace': u'\u2005',
    u'bullet': u'\u2022',
    u'lquote': u'\u2018',
    u'rquote': u'\u2019',
    u'ldblquote': u'\u201C',
    u'rdblquote': u'\u201D',
    u'ltrmark': u'\u200E',
    u'rtlmark': u'\u200F',
    u'zwj': u'\u200D',
    u'zwnj': u'\u200C'}
CHARSET_MAPPING = {
    u'fcharset0': u'cp1252',
    u'fcharset161': u'cp1253',
    u'fcharset162': u'cp1254',
    u'fcharset163': u'cp1258',
    u'fcharset177': u'cp1255',
    u'fcharset178': u'cp1256',
    u'fcharset186': u'cp1257',
    u'fcharset204': u'cp1251',
    u'fcharset222': u'cp874',
    u'fcharset238': u'cp1250'}


class VerseType(object):
    """
    VerseType provides an enumeration for the tags that may be associated
    with verses in songs.
    """
    Verse = 0
    Chorus = 1
    Bridge = 2
    PreChorus = 3
    Intro = 4
    Ending = 5
    Other = 6

    Names = [
        u'Verse',
        u'Chorus',
        u'Bridge',
        u'Pre-Chorus',
        u'Intro',
        u'Ending',
        u'Other']
    Tags = [name[0].lower() for name in Names]

    TranslatedNames = [
        unicode(translate('SongsPlugin.VerseType', 'Verse')),
        unicode(translate('SongsPlugin.VerseType', 'Chorus')),
        unicode(translate('SongsPlugin.VerseType', 'Bridge')),
        unicode(translate('SongsPlugin.VerseType', 'Pre-Chorus')),
        unicode(translate('SongsPlugin.VerseType', 'Intro')),
        unicode(translate('SongsPlugin.VerseType', 'Ending')),
        unicode(translate('SongsPlugin.VerseType', 'Other'))]
    TranslatedTags = [name[0].lower() for name in TranslatedNames]

    @staticmethod
    def translated_tag(verse_tag, default=Other):
        """
        Return the translated UPPERCASE tag for a given tag,
        used to show translated verse tags in UI

        ``verse_tag``
            The string to return a VerseType for

        ``default``
            Default return value if no matching tag is found
        """
        verse_tag = verse_tag[0].lower()
        for num, tag in enumerate(VerseType.Tags):
            if verse_tag == tag:
                return VerseType.TranslatedTags[num].upper()
        if default in VerseType.TranslatedTags:
            return VerseType.TranslatedTags[default].upper()

    @staticmethod
    def translated_name(verse_tag, default=Other):
        """
        Return the translated name for a given tag

        ``verse_tag``
            The string to return a VerseType for

        ``default``
            Default return value if no matching tag is found
        """
        verse_tag = verse_tag[0].lower()
        for num, tag in enumerate(VerseType.Tags):
            if verse_tag == tag:
                return VerseType.TranslatedNames[num]
        if default in VerseType.TranslatedNames:
            return VerseType.TranslatedNames[default]

    @staticmethod
    def from_tag(verse_tag, default=Other):
        """
        Return the VerseType for a given tag

        ``verse_tag``
            The string to return a VerseType for

        ``default``
            Default return value if no matching tag is found
        """
        verse_tag = verse_tag[0].lower()
        for num, tag in enumerate(VerseType.Tags):
            if verse_tag == tag:
                return num
        return default

    @staticmethod
    def from_translated_tag(verse_tag, default=Other):
        """
        Return the VerseType for a given tag

        ``verse_tag``
            The string to return a VerseType for

        ``default``
            Default return value if no matching tag is found
        """
        verse_tag = verse_tag[0].lower()
        for num, tag in enumerate(VerseType.TranslatedTags):
            if verse_tag == tag:
                return num
        return default

    @staticmethod
    def from_string(verse_name, default=Other):
        """
        Return the VerseType for a given string

        ``verse_name``
            The string to return a VerseType for

        ``default``
            Default return value if no matching tag is found
        """
        verse_name = verse_name.lower()
        for num, name in enumerate(VerseType.Names):
            if verse_name == name.lower():
                return num
        return default

    @staticmethod
    def from_translated_string(verse_name):
        """
        Return the VerseType for a given string

        ``verse_name``
            The string to return a VerseType for
        """
        verse_name = verse_name.lower()
        for num, translation in enumerate(VerseType.TranslatedNames):
            if verse_name == translation.lower():
                return num

    @staticmethod
    def from_loose_input(verse_name, default=Other):
        """
        Return the VerseType for a given string

        ``verse_name``
            The string to return a VerseType for

        ``default``
            Default return value if no matching tag is found
        """
        if len(verse_name) > 1:
            verse_index = VerseType.from_translated_string(verse_name)
            if verse_index is None:
                verse_index = VerseType.from_string(verse_name, default)
        elif len(verse_name) == 1:
            verse_index = VerseType.from_translated_tag(verse_name, None)
            if verse_index is None:
                verse_index = VerseType.from_tag(verse_name, default)
        else:
            return default
        return verse_index


def retrieve_windows_encoding(recommendation=None):
    """
    Determines which encoding to use on an information source. The process uses
    both automated detection, which is passed to this method as a
    recommendation, and user confirmation to return an encoding.

    ``recommendation``
        A recommended encoding discovered programmatically for the user to
        confirm.
    """
    # map chardet result to compatible windows standard code page
    codepage_mapping = {'IBM866': u'cp866', 'TIS-620': u'cp874',
        'SHIFT_JIS': u'cp932', 'GB2312': u'cp936', 'HZ-GB-2312': u'cp936',
        'EUC-KR': u'cp949', 'Big5': u'cp950', 'ISO-8859-2': u'cp1250',
        'windows-1250': u'cp1250', 'windows-1251': u'cp1251',
        'windows-1252': u'cp1252', 'ISO-8859-7': u'cp1253',
        'windows-1253': u'cp1253', 'ISO-8859-8': u'cp1255',
        'windows-1255': u'cp1255'}
    if recommendation in codepage_mapping:
        recommendation = codepage_mapping[recommendation]

    # Show dialog for encoding selection
    encodings = [(u'cp1256', translate('SongsPlugin', 'Arabic (CP-1256)')),
        (u'cp1257', translate('SongsPlugin', 'Baltic (CP-1257)')),
        (u'cp1250', translate('SongsPlugin', 'Central European (CP-1250)')),
        (u'cp1251', translate('SongsPlugin', 'Cyrillic (CP-1251)')),
        (u'cp1253', translate('SongsPlugin', 'Greek (CP-1253)')),
        (u'cp1255', translate('SongsPlugin', 'Hebrew (CP-1255)')),
        (u'cp932', translate('SongsPlugin', 'Japanese (CP-932)')),
        (u'cp949', translate('SongsPlugin', 'Korean (CP-949)')),
        (u'cp936', translate('SongsPlugin', 'Simplified Chinese (CP-936)')),
        (u'cp874', translate('SongsPlugin', 'Thai (CP-874)')),
        (u'cp950', translate('SongsPlugin', 'Traditional Chinese (CP-950)')),
        (u'cp1254', translate('SongsPlugin', 'Turkish (CP-1254)')),
        (u'cp1258', translate('SongsPlugin', 'Vietnam (CP-1258)')),
        (u'cp1252', translate('SongsPlugin', 'Western European (CP-1252)'))]
    recommended_index = -1
    if recommendation:
        for index in range(len(encodings)):
            if recommendation == encodings[index][0]:
                recommended_index = index
                break
    if recommended_index > 0:
        choice = QtGui.QInputDialog.getItem(None,
            translate('SongsPlugin', 'Character Encoding'),
            translate('SongsPlugin', 'The codepage setting is responsible\n'
                'for the correct character representation.\n'
                'Usually you are fine with the preselected choice.'),
            [pair[1] for pair in encodings], recommended_index, False)
    else:
        choice = QtGui.QInputDialog.getItem(None,
            translate('SongsPlugin', 'Character Encoding'),
            translate('SongsPlugin', 'Please choose the character encoding.\n'
                'The encoding is responsible for the correct character '
                'representation.'), [pair[1] for pair in encodings], 0, False)
    if not choice[1]:
        return None
    return filter(lambda item: item[1] == choice[0], encodings)[0][0]


def clean_string(string):
    """
    Strips punctuation from the passed string to assist searching
    """
    return WHITESPACE.sub(u' ', APOSTROPHE.sub(u'', string)).lower()


def clean_title(title):
    """
    Cleans the song title by removing Unicode control chars groups C0 & C1,
    as well as any trailing spaces
    """
    return CONTROL_CHARS.sub(u'', title).rstrip()


def clean_song(manager, song):
    """
    Cleans the search title, rebuilds the search lyrics, adds a default author
    if the song does not have one and other clean ups. This should always
    called when a new song is added or changed.

    ``manager``
        The song's manager.

    ``song``
        The song object.
    """
    if isinstance(song.title, buffer):
        song.title = unicode(song.title)
    if isinstance(song.alternate_title, buffer):
        song.alternate_title = unicode(song.alternate_title)
    if isinstance(song.lyrics, buffer):
        song.lyrics = unicode(song.lyrics)
    if song.title:
        song.title = clean_title(song.title)
    else:
        song.title = u''
    if song.alternate_title:
        song.alternate_title = clean_title(song.alternate_title)
    else:
        song.alternate_title = u''
    song.search_title = clean_string(song.title) + u'@' + \
        clean_string(song.alternate_title)
    # Only do this, if we the song is a 1.9.4 song (or older).
    if song.lyrics.find(u'<lyrics language="en">') != -1:
        # Remove the old "language" attribute from lyrics tag (prior to 1.9.5).
        # This is not very important, but this keeps the database clean. This
        # can be removed when everybody has cleaned his songs.
        song.lyrics = song.lyrics.replace(
            u'<lyrics language="en">', u'<lyrics>')
        verses = SongXML().get_verses(song.lyrics)
        song.search_lyrics = u' '.join([clean_string(verse[1])
            for verse in verses])
        # We need a new and clean SongXML instance.
        sxml = SongXML()
        # Rebuild the song's verses, to remove any wrong verse names (for
        # example translated ones), which might have been added prior to 1.9.5.
        # List for later comparison.
        compare_order = []
        for verse in verses:
            verse_type = VerseType.Tags[VerseType.from_loose_input(
                verse[0][u'type'])]
            sxml.add_verse_to_lyrics(
                verse_type,
                verse[0][u'label'],
                verse[1],
                verse[0].get(u'lang')
            )
            compare_order.append((u'%s%s' % (verse_type, verse[0][u'label'])
                ).upper())
            if verse[0][u'label'] == u'1':
                compare_order.append(verse_type.upper())
        song.lyrics = unicode(sxml.extract_xml(), u'utf-8')
        # Rebuild the verse order, to convert translated verse tags, which might
        # have been added prior to 1.9.5.
        if song.verse_order:
            order = CONTROL_CHARS.sub(u'', song.verse_order).strip().split()
        else:
            order = []
        new_order = []
        for verse_def in order:
            verse_type = VerseType.Tags[
                VerseType.from_loose_input(verse_def[0])]
            if len(verse_def) > 1:
                new_order.append(
                    (u'%s%s' % (verse_type, verse_def[1:])).upper())
            else:
                new_order.append(verse_type.upper())
        song.verse_order = u' '.join(new_order)
        # Check if the verse order contains tags for verses which do not exist.
        for order in new_order:
            if order not in compare_order:
                song.verse_order = u''
                break
    else:
        verses = SongXML().get_verses(song.lyrics)
        song.search_lyrics = u' '.join([clean_string(verse[1])
            for verse in verses])

    # The song does not have any author, add one.
    if not song.authors:
        name = SongStrings.AuthorUnknown
        author = manager.get_object_filtered(
            Author, Author.display_name == name)
        if author is None:
            author = Author.populate(
                display_name=name, last_name=u'', first_name=u'')
        song.authors.append(author)
    if song.copyright:
        song.copyright = CONTROL_CHARS.sub(u'', song.copyright).strip()


def get_encoding(font, font_table, default_encoding, failed=False):
    """
    Finds an encoding to use. Asks user, if necessary.

    ``font``
    The number of currently active font.

    ``font_table``
    Dictionary of fonts and respective encodings.

    ``default_encoding``
    The default encoding to use when font_table is empty or no font is used.

    ``failed``
    A boolean indicating whether the previous encoding didn't work.
    """
    encoding = None
    if font in font_table:
        encoding = font_table[font]
    if not encoding and default_encoding:
        encoding = default_encoding
    if not encoding or failed:
        encoding = retrieve_windows_encoding()
        default_encoding = encoding
    font_table[font] = encoding
    return encoding, default_encoding


def strip_rtf(text, default_encoding=None):
    """
    This function strips RTF control structures and returns an unicode string.

    Thanks to Markus Jarderot (MizardX) for this code, used by permission.
    http://stackoverflow.com/questions/188545

    ``text``
    RTF-encoded text, a string.

    ``default_encoding``
    Default encoding to use when no encoding is specified.
    """
    # Current font is the font tag we last met.
    font = u''
    # Character encoding is defined inside fonttable.
    # font_table could contain eg u'0': u'cp1252'
    font_table = {u'': u''}
    # Stack of things to keep track of when entering/leaving groups.
    stack = []
    # Whether this group (and all inside it) are "ignorable".
    ignorable = False
    # Number of ASCII characters to skip after an unicode character.
    ucskip = 1
    # Number of ASCII characters left to skip.
    curskip = 0
    # Output buffer.
    out = []
    for match in PATTERN.finditer(text):
        word, arg, hex, char, brace, tchar = match.groups()
        if brace:
            curskip = 0
            if brace == u'{':
                # Push state
                stack.append((ucskip, ignorable, font))
            elif brace == u'}':
                # Pop state
                ucskip, ignorable, font = stack.pop()
        # \x (not a letter)
        elif char:
            curskip = 0
            if char == u'~' and not ignorable:
                out.append(u'\xA0')
            elif char in u'{}\\' and not ignorable:
                out.append(char)
            elif char == u'-' and not ignorable:
                out.append(u'\u00AD')
            elif char == u'_' and not ignorable:
                out.append(u'\u2011')
            elif char == u'*':
                ignorable = True
        # \command
        elif word:
            curskip = 0
            if word in DESTINATIONS:
                ignorable = True
            elif word in SPECIAL_CHARS:
                out.append(SPECIAL_CHARS[word])
            elif word == u'uc':
                ucskip = int(arg)
            elif word == u' ':
                c = int(arg)
                if c < 0:
                    c += 0x10000
                out.append(unichr(c))
                curskip = ucskip
            elif word == u'fonttbl':
                ignorable = True
            elif word == u'f':
                font = arg
            elif word == u'ansicpg':
                font_table[font] = 'cp' + arg
            elif word == u'fcharset' and font not in font_table and \
                word + arg in CHARSET_MAPPING:
                # \ansicpg overrides \fcharset, if present.
                font_table[font] = CHARSET_MAPPING[word + arg]
        # \'xx
        elif hex:
            if curskip > 0:
                curskip -= 1
            elif not ignorable:
                charcode = int(hex, 16)
                failed = False
                while True:
                    try:
                        encoding, default_encoding = get_encoding(font,
                            font_table, default_encoding, failed=failed)
                        if not encoding:
                            return None
                        out.append(chr(charcode).decode(encoding))
                    except UnicodeDecodeError:
                        failed = True
                    else:
                        break
        elif tchar:
            if curskip > 0:
                curskip -= 1
            elif not ignorable:
                out.append(tchar)
    text = u''.join(out)
    return text, default_encoding

def natcmp(a, b):
    """
    Natural string comparison which mimics the behaviour of Python's internal
    cmp function.
    """
    if len(a) <= len(b):
        for i, key in enumerate(a):
            if isinstance(key, int) and isinstance(b[i], int):
                result = cmp(key, b[i])
            elif isinstance(key, int) and not isinstance(b[i], int):
                result = locale_direct_compare(QtCore.QString(str(key)), b[i])
            elif not isinstance(key, int) and isinstance(b[i], int):
                result = locale_direct_compare(key, QtCore.QString(str(b[i])))
            else:
                result = locale_direct_compare(key, b[i])
            if result != 0:
                return result
        if len(a) == len(b):
            return 0
        else:
            return -1
    else:
        for i, key in enumerate(b):
            if isinstance(a[i], int) and isinstance(key, int):
                result = cmp(a[i], key)
            elif isinstance(a[i], int) and not isinstance(key, int):
                result = locale_direct_compare(QtCore.QString(str(a[i])), key)
            elif not isinstance(a[i], int) and isinstance(key, int):
                result = locale_direct_compare(a[i], QtCore.QString(str(key)))
            else:
                result = locale_direct_compare(a[i], key)
            if result != 0:
                return result
        return 1

from xml import OpenLyrics, SongXML
from songstab import SongsTab
from mediaitem import SongMediaItem



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
Extend QListWidget to handle drag and drop functionality
"""
import os

from PyQt4 import QtCore, QtGui

from openlp.core.lib import Receiver

class ListWidgetWithDnD(QtGui.QListWidget):
    """
    Provide a list widget to store objects and handle drag and drop events
    """
    def __init__(self, parent=None, name=u''):
        """
        Initialise the list widget
        """
        QtGui.QListWidget.__init__(self, parent)
        self.mimeDataText = name
        assert(self.mimeDataText)

    def activateDnD(self):
        """
        Activate DnD of widget
        """
        self.setAcceptDrops(True)
        self.setDragDropMode(QtGui.QAbstractItemView.DragDrop)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'%s_dnd' % self.mimeDataText),
            self.parent().loadFile)

    def mouseMoveEvent(self, event):
        """
        Drag and drop event does not care what data is selected
        as the recipient will use events to request the data move
        just tell it what plugin to call
        """
        if event.buttons() != QtCore.Qt.LeftButton:
            event.ignore()
            return
        if not self.selectedItems():
            event.ignore()
            return
        drag = QtGui.QDrag(self)
        mimeData = QtCore.QMimeData()
        drag.setMimeData(mimeData)
        mimeData.setText(self.mimeDataText)
        drag.start(QtCore.Qt.CopyAction)

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()
        else:
            event.ignore()

    def dragMoveEvent(self, event):
        if event.mimeData().hasUrls():
            event.setDropAction(QtCore.Qt.CopyAction)
            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):
        """
        Receive drop event check if it is a file and process it if it is.

        ``event``
            Handle of the event pint passed
        """
        if event.mimeData().hasUrls():
            event.setDropAction(QtCore.Qt.CopyAction)
            event.accept()
            files = []
            for url in event.mimeData().urls():
                localFile = unicode(url.toLocalFile())
                if os.path.isfile(localFile):
                    files.append(localFile)
                elif os.path.isdir(localFile):
                    listing = os.listdir(localFile)
                    for file in listing:
                        files.append(os.path.join(localFile, file))
            Receiver.send_message(u'%s_dnd' % self.mimeDataText, files)
        else:
            event.ignore()



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from openlp.core.lib import translate, build_icon
from openlp.core.lib.ui import UiStrings, create_button_box


class Ui_FormattingTagDialog(object):

    def setupUi(self, formattingTagDialog):
        formattingTagDialog.setObjectName(u'formattingTagDialog')
        formattingTagDialog.setWindowIcon(build_icon(u':/icon/openlp-logo.svg'))
        formattingTagDialog.resize(725, 548)
        self.listdataGridLayout = QtGui.QGridLayout(formattingTagDialog)
        self.listdataGridLayout.setMargin(8)
        self.listdataGridLayout.setObjectName(u'listdataGridLayout')
        self.tagTableWidget = QtGui.QTableWidget(formattingTagDialog)
        self.tagTableWidget.setHorizontalScrollBarPolicy(
            QtCore.Qt.ScrollBarAlwaysOff)
        self.tagTableWidget.setEditTriggers(
            QtGui.QAbstractItemView.NoEditTriggers)
        self.tagTableWidget.setAlternatingRowColors(True)
        self.tagTableWidget.setSelectionMode(
            QtGui.QAbstractItemView.SingleSelection)
        self.tagTableWidget.setSelectionBehavior(
            QtGui.QAbstractItemView.SelectRows)
        self.tagTableWidget.setCornerButtonEnabled(False)
        self.tagTableWidget.setObjectName(u'tagTableWidget')
        self.tagTableWidget.setColumnCount(4)
        self.tagTableWidget.setRowCount(0)
        self.tagTableWidget.horizontalHeader().setStretchLastSection(True)
        item = QtGui.QTableWidgetItem()
        self.tagTableWidget.setHorizontalHeaderItem(0, item)
        item = QtGui.QTableWidgetItem()
        self.tagTableWidget.setHorizontalHeaderItem(1, item)
        item = QtGui.QTableWidgetItem()
        self.tagTableWidget.setHorizontalHeaderItem(2, item)
        item = QtGui.QTableWidgetItem()
        self.tagTableWidget.setHorizontalHeaderItem(3, item)
        self.listdataGridLayout.addWidget(self.tagTableWidget, 0, 0, 1, 1)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(u'horizontalLayout')
        spacerItem = QtGui.QSpacerItem(40, 20,
            QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.deletePushButton = QtGui.QPushButton(formattingTagDialog)
        self.deletePushButton.setObjectName(u'deletePushButton')
        self.horizontalLayout.addWidget(self.deletePushButton)
        self.listdataGridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)
        self.editGroupBox = QtGui.QGroupBox(formattingTagDialog)
        self.editGroupBox.setObjectName(u'editGroupBox')
        self.dataGridLayout = QtGui.QGridLayout(self.editGroupBox)
        self.dataGridLayout.setObjectName(u'dataGridLayout')
        self.descriptionLabel = QtGui.QLabel(self.editGroupBox)
        self.descriptionLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.descriptionLabel.setObjectName(u'descriptionLabel')
        self.dataGridLayout.addWidget(self.descriptionLabel, 0, 0, 1, 1)
        self.descriptionLineEdit = QtGui.QLineEdit(self.editGroupBox)
        self.descriptionLineEdit.setObjectName(u'descriptionLineEdit')
        self.dataGridLayout.addWidget(self.descriptionLineEdit, 0, 1, 2, 1)
        self.newPushButton = QtGui.QPushButton(self.editGroupBox)
        self.newPushButton.setObjectName(u'newPushButton')
        self.dataGridLayout.addWidget(self.newPushButton, 0, 2, 2, 1)
        self.tagLabel = QtGui.QLabel(self.editGroupBox)
        self.tagLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.tagLabel.setObjectName(u'tagLabel')
        self.dataGridLayout.addWidget(self.tagLabel, 2, 0, 1, 1)
        self.tagLineEdit = QtGui.QLineEdit(self.editGroupBox)
        self.tagLineEdit.setMaximumSize(QtCore.QSize(50, 16777215))
        self.tagLineEdit.setMaxLength(5)
        self.tagLineEdit.setObjectName(u'tagLineEdit')
        self.dataGridLayout.addWidget(self.tagLineEdit, 2, 1, 1, 1)
        self.startTagLabel = QtGui.QLabel(self.editGroupBox)
        self.startTagLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.startTagLabel.setObjectName(u'startTagLabel')
        self.dataGridLayout.addWidget(self.startTagLabel, 3, 0, 1, 1)
        self.startTagLineEdit = QtGui.QLineEdit(self.editGroupBox)
        self.startTagLineEdit.setObjectName(u'startTagLineEdit')
        self.dataGridLayout.addWidget(self.startTagLineEdit, 3, 1, 1, 1)
        self.endTagLabel = QtGui.QLabel(self.editGroupBox)
        self.endTagLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.endTagLabel.setObjectName(u'endTagLabel')
        self.dataGridLayout.addWidget(self.endTagLabel, 4, 0, 1, 1)
        self.endTagLineEdit = QtGui.QLineEdit(self.editGroupBox)
        self.endTagLineEdit.setObjectName(u'endTagLineEdit')
        self.dataGridLayout.addWidget(self.endTagLineEdit, 4, 1, 1, 1)
        self.savePushButton = QtGui.QPushButton(self.editGroupBox)
        self.savePushButton.setObjectName(u'savePushButton')
        self.dataGridLayout.addWidget(self.savePushButton, 4, 2, 1, 1)
        self.listdataGridLayout.addWidget(self.editGroupBox, 2, 0, 1, 1)
        self.buttonBox = create_button_box(formattingTagDialog, 'buttonBox',
            [u'close'])
        self.listdataGridLayout.addWidget(self.buttonBox, 3, 0, 1, 1)

        self.retranslateUi(formattingTagDialog)

    def retranslateUi(self, formattingTagDialog):
        formattingTagDialog.setWindowTitle(translate(
            'OpenLP.FormattingTagDialog', 'Configure Formatting Tags'))
        self.editGroupBox.setTitle(
            translate('OpenLP.FormattingTagDialog', 'Edit Selection'))
        self.savePushButton.setText(
            translate('OpenLP.FormattingTagDialog', 'Save'))
        self.descriptionLabel.setText(
            translate('OpenLP.FormattingTagDialog', 'Description'))
        self.tagLabel.setText(translate('OpenLP.FormattingTagDialog', 'Tag'))
        self.startTagLabel.setText(
            translate('OpenLP.FormattingTagDialog', 'Start HTML'))
        self.endTagLabel.setText(
            translate('OpenLP.FormattingTagDialog', 'End HTML'))
        self.deletePushButton.setText(UiStrings().Delete)
        self.newPushButton.setText(UiStrings().New)
        self.tagTableWidget.horizontalHeaderItem(0).setText(
            translate('OpenLP.FormattingTagDialog', 'Description'))
        self.tagTableWidget.horizontalHeaderItem(1).setText(
            translate('OpenLP.FormattingTagDialog', 'Tag'))
        self.tagTableWidget.horizontalHeaderItem(2).setText(
            translate('OpenLP.FormattingTagDialog', 'Start HTML'))
        self.tagTableWidget.horizontalHeaderItem(3).setText(
            translate('OpenLP.FormattingTagDialog', 'End HTML'))
        self.tagTableWidget.setColumnWidth(0, 120)
        self.tagTableWidget.setColumnWidth(1, 80)
        self.tagTableWidget.setColumnWidth(2, 330)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import re
import logging

from PyQt4 import QtCore, QtGui

from openlp.core.lib.ui import critical_error_message_box
from openlp.plugins.songs.lib import VerseType, translate

from editversedialog import Ui_EditVerseDialog

log = logging.getLogger(__name__)

VERSE_REGEX = re.compile(r'---\[(.+):\D*(\d*)\D*.*\]---')

class EditVerseForm(QtGui.QDialog, Ui_EditVerseDialog):
    """
    This is the form that is used to edit the verses of the song.
    """
    def __init__(self, parent=None):
        """
        Constructor
        """
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        QtCore.QObject.connect(self.verseTextEdit,
            QtCore.SIGNAL('customContextMenuRequested(QPoint)'),
            self.contextMenu)
        QtCore.QObject.connect(self.insertButton, QtCore.SIGNAL(u'clicked()'),
            self.onInsertButtonClicked)
        QtCore.QObject.connect(self.splitButton, QtCore.SIGNAL(u'clicked()'),
            self.onSplitButtonClicked)
        QtCore.QObject.connect(self.verseTextEdit,
            QtCore.SIGNAL(u'cursorPositionChanged()'),
            self.onCursorPositionChanged)
        QtCore.QObject.connect(self.verseTypeComboBox,
            QtCore.SIGNAL(u'currentIndexChanged(int)'),
            self.onVerseTypeComboBoxChanged)

    def contextMenu(self, point):
        item = self.serviceManagerList.itemAt(point)

    def insertVerse(self, verse_tag, verse_num=1):
        if self.verseTextEdit.textCursor().columnNumber() != 0:
            self.verseTextEdit.insertPlainText(u'\n')
        verse_tag = VerseType.translated_name(verse_tag)
        self.verseTextEdit.insertPlainText(u'---[%s:%s]---\n' %
            (verse_tag, verse_num))
        self.verseTextEdit.setFocus()

    def onSplitButtonClicked(self):
        text = self.verseTextEdit.toPlainText()
        position = self.verseTextEdit.textCursor().position()
        insert_string = u'[---]'
        if position and text[position-1] != u'\n':
             insert_string = u'\n' + insert_string
        if position ==  len(text) or text[position] != u'\n':
             insert_string += u'\n'
        self.verseTextEdit.insertPlainText(insert_string)
        self.verseTextEdit.setFocus()

    def onInsertButtonClicked(self):
        verse_type_index = self.verseTypeComboBox.currentIndex()
        self.insertVerse(VerseType.Tags[verse_type_index],
            self.verseNumberBox.value())

    def onVerseTypeComboBoxChanged(self):
        """
        Adjusts the verse number SpinBox in regard to the selected verse type
        and the cursor's position.
        """
        position = self.verseTextEdit.textCursor().position()
        text = unicode(self.verseTextEdit.toPlainText())
        verse_name = VerseType.TranslatedNames[
            self.verseTypeComboBox.currentIndex()]
        if not text:
            return
        position = text.rfind(u'---[%s' % verse_name, 0, position)
        if position == -1:
            self.verseNumberBox.setValue(1)
            return
        text = text[position:]
        position = text.find(u']---')
        if position == -1:
            return
        text = text[:position + 4]
        match = VERSE_REGEX.match(text)
        if match:
            verse_tag = match.group(1)
            try:
                verse_num = int(match.group(2))
            except ValueError:
                verse_num = 1
            if VerseType.from_loose_input(verse_tag, False):
                self.verseNumberBox.setValue(verse_num)

    def onCursorPositionChanged(self):
        """
        Determines the previous verse type and number in regard to the cursor's
        position and adjusts the ComboBox and SpinBox to these values.
        """
        position = self.verseTextEdit.textCursor().position()
        text = unicode(self.verseTextEdit.toPlainText())
        if not text:
            return
        if text.rfind(u'[', 0, position) > text.rfind(u']', 0, position) and \
            text.find(u']', position) < text.find(u'[', position):
            return
        position = text.rfind(u'---[', 0, position)
        if position == -1:
            return
        text = text[position:]
        position = text.find(u']---')
        if position == -1:
            return
        text = text[:position + 4]
        match = VERSE_REGEX.match(text)
        if match:
            verse_type = match.group(1)
            verse_type_index = VerseType.from_loose_input(verse_type, None)
            try:
                verse_number = int(match.group(2))
            except ValueError:
                verse_number = 1
            if verse_type_index is not None:
                self.verseTypeComboBox.setCurrentIndex(verse_type_index)
                self.verseNumberBox.setValue(verse_number)

    def setVerse(self, text, single=False,
        tag=u'%s1' % VerseType.Tags[VerseType.Verse]):
        self.hasSingleVerse = single
        if single:
            verse_type_index = VerseType.from_tag(tag[0], None)
            verse_number = tag[1:]
            if verse_type_index is not None:
                self.verseTypeComboBox.setCurrentIndex(verse_type_index)
            self.verseNumberBox.setValue(int(verse_number))
            self.insertButton.setVisible(False)
        else:
            if not text:
                text = u'---[%s:1]---\n' % \
                    VerseType.TranslatedNames[VerseType.Verse]
            self.verseTypeComboBox.setCurrentIndex(0)
            self.verseNumberBox.setValue(1)
            self.insertButton.setVisible(True)
        self.verseTextEdit.setPlainText(text)
        self.verseTextEdit.setFocus(QtCore.Qt.OtherFocusReason)
        self.verseTextEdit.moveCursor(QtGui.QTextCursor.End)

    def getVerse(self):
        return self.verseTextEdit.toPlainText(), \
            VerseType.Tags[self.verseTypeComboBox.currentIndex()], \
            unicode(self.verseNumberBox.value())

    def getVerseAll(self):
        text = self.verseTextEdit.toPlainText()
        if not text.startsWith(u'---['):
            text = u'---[%s:1]---\n%s' % \
                (VerseType.TranslatedNames[VerseType.Verse], text)
        return text
